<?php
namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00001000ItemLancamentoPropostaFactory;
use Teknisa\SM\Factory\Sm00001000LancamentoPropostaFactory;
use Teknisa\SM\Factory\SmPercentualMaximoDescontoAcrescimoOperadorFactory;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Service\Sm00001000ItemFormPrecoPropostaService;


class Sm00001000ItemLancamentoPropostaService {
    
    protected $itensFactory;
    protected $propostaFactory;
    protected $descontoAcrescimoFactory;
    protected $propostaService;
    protected $newCode;
    protected $entityManager;
    protected $itFormacaoService;
        
    public function __construct(Sm00001000ItemLancamentoPropostaFactory $itensFactory, Sm00001000LancamentoPropostaFactory $propostaFactory, SmPercentualMaximoDescontoAcrescimoOperadorFactory $descontoAcrescimoFactory, Sm00001000LancamentoPropostaService $propostaService, NewCode $newCode, EntityManager $entityManager, Sm00001000ItemFormPrecoPropostaService $itFormacaoService) {
        $this->itensFactory             = $itensFactory;
        $this->propostaFactory          = $propostaFactory;
        $this->descontoAcrescimoFactory = $descontoAcrescimoFactory;
        $this->propostaService          = $propostaService;
        $this->newCode                  = $newCode;
        $this->entityManager            = $entityManager;
        $this->itFormacaoService        = $itFormacaoService;
    }
     /**
     *Valida se o percentul dado pelo o vendedor esta conforme com seu perfil de acesso ou operador
     */
    public function validaPorcentualDescontoAcrescimoSave($cdservorca, $cdoperador, $idtipo, $nrorg) {
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "CDSERVORCA" => $cdservorca,
            "IDTIPO"     => $idtipo,
            "NRORG"      => $nrorg
        );
        
        $desconto = $this->descontoAcrescimoFactory->getPercentualDescontoAcrescimo($params);
        return (count($desconto) > 0) ? $desconto[0]: null;
    }
    
    /**
     * Atualiza a quantidade de um item da tabela de agendamentos pendentes 
     * caso ele seja alterado na proposta
    **/
    public function atualizaQuantidadeItensAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg, $quantidade){
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRITEMORCAMENTO" => $nritemorcamento,
            "QTPENDENCIA"     => $quantidade,
            "NRORG"           => $nrorg
        );
        
        $this->itensFactory->atualizaQuantidadeItensAgendPendente($params);
    }
    
    /**
     * Exclui um item da tabela de agendamentos pendentes caso ele seja excluido na proposta
    **/
    public function excluiItemAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        
        $this->itensFactory->excluiItemAgendPendente($params);
    }
    /**
     * Pega uma alteração realizada no valor dos itens de tipo elemento
     * e replica para o elemento raiz
    **/
    public function replicaAteracaoParaItemPai($nrseqorcamento, $nrorg, $cdoperador, $cdservorca){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "CDSERVORCA"     => $cdservorca,
            "NRORG"          => $nrorg,
        );
        
        $this->itensFactory->replicaAteracaoParaItemPai($params, $cdoperador);
    }
    /**
     * Retorna a quantidade de elementos de uma composição de serviço
    **/
    public function getCountElementos($nrtipoorcamento, $nrseqorcamento, $idelementopai, $cdtpmedidaserv, $nrorg, $cdservorca){
         $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "CDSERVORCA"      => $cdservorca,
            "NRORG"           => $nrorg,
            "IDELEMENTOPAI"   => $idelementopai,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "CDTPMEDIDASERV"  => $cdtpmedidaserv
        );
        
        return $this->itensFactory->getCountElementos($params);
    }
    /**
     * Faz o calculo do ajuste do elemento ou item
    **/
    public function getVrtotalComAjuste($idtpdescorca, $vrpercdescorca, $vrunitario, $qtmedida, $countElemento){
        if($idtpdescorca == "N"){
            $vrtotal = ($vrunitario * $qtmedida);
        } else {
            if($idtpdescorca == 'V' || $idtpdescorca == 'X') 
                $vrdescorca = $vrpercdescorca * ($idtpdescorca == 'X' ? -1 : 1);
            else 
                $vrdescorca = ($vrunitario * $qtmedida) * ($vrpercdescorca/100) * ($idtpdescorca == 'Y' ? -1 : 1);
            $vrtotal = ($vrunitario * $qtmedida) - $vrdescorca;
        }
        
        return $vrtotal;
    }
    /**
     * Atualiza o valor dos itens de composição
    **/
    public function atualizarValorItemProposta($nrseqorcamento, $nrorg, $nritemorcamento, $cdservorca, $vrunitario, $qtmedida, $idtpdescorca, $vrpercdescorca, $vrunittblpreco){
        $params = array(
            "QTMEDIDAORCA"           => $qtmedida,
            "VRUNITITEMORCATBLPRECO" => $vrunittblpreco,
            "NRSEQORCAMENTO"         => $nrseqorcamento,
            "NRITEMORCAMENTO"        => $nritemorcamento,
            "CDSERVORCA"             => $cdservorca,
            "NRORG"                  => $nrorg,
            "VRITEMSEMDESC"          => $vrunitario * $qtmedida,
            "VRTOTALITEMORCA"        => $this->getVrtotalComAjuste($idtpdescorca, $vrpercdescorca, $vrunitario, $qtmedida, 1)
        );
        $this->itensFactory->atualizarValorItemProposta($params);
    }
    /**
     * Muda o valor unitario de um elemento
    **/
    public function atualizarPrecoElementoItem($nrseqorcamento, $nrorg, $item){
        $paramElemento = array(
            "CDSERVORCA" => $item["CDSERVORCARELAC"],
            "IDELEMENTO" => $item["IDELEMENTO"]
        );
        $elementos = $this->itensFactory->getElementosComposicao($paramElemento);
        foreach($elementos as $elemento){
            if($item["IDMEDIDAFIXA"] == 'S') {
                $vrunitario = $item["VRPRECO"];
                $tpmedida   = $item["CDTPMEDIDASERV"]; 
            } else { 
                $vrunitario = $item["VRPRECO"] * $elemento["QTMEDIDASERV"];
                $tpmedida   = $item["CDTPMEDIDAVARIAVEL"];
            }
            
            $countElemento = $this->getCountElementos($item["NRTIPOORCAMENTO"], $nrseqorcamento, $item["IDELEMENTO"], $item["CDTPMEDIDASERV"], $nrorg, $item["CDSERVORCARELAC"]);
            $countElemento["QTELEMENTOS"] = ($countElemento["QTELEMENTOS"] == 0) ? 1 : $countElemento["QTELEMENTOS"];
            $qtmedida      = $item["QTMEDIDAORCA"]/$countElemento["QTELEMENTOS"];
            $vrajuste      = $this->getVrtotalComAjuste($item["IDTPDESCORCA"], $item["VRPERCDESCORCA"], $vrunitario, $qtmedida, $countElemento["QTELEMENTOS"]);
            $params = array(
                "NRSEQORCAMENTO"  => $nrseqorcamento,
                "CDSERVORCA"      => $item["CDSERVORCARELAC"],
                "IDELEMENTO"      => $elemento["IDELEMENTO"],
                "IDELEMENTOPAI"   => $elemento["IDELEMENTOPAI"],
                "VRUNITSERVORCA"  => $vrunitario,
                "VRTOTSAJUSTE"    => $vrunitario * $qtmedida,
                "QTMEDSERVORCA"   => $qtmedida,
                "VRTOTSERVORCA"   => $vrajuste,
                "CDTPMEDIDASERV"  => $tpmedida,
                "NRORG"           => $nrorg
            );
            $this->itensFactory->atualizarPrecoElementoItem($params);
        }
    }
    
    /**
     * Faz a validação do valor dos itens de um orçamento 
     ***/
    public function validarValoresItensProp($nrseqorcamento, $nrorg, $cdoperador){
        $validacao = false;
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $infoItens  = $this->itensFactory->getInfoValoresItensOrcamento($params);
        foreach($infoItens as $item){
            if($item["VRPRECO"] != 0) {
                $vrtotal = $item["VRUNITITEMORCATBLPRECO"] * $item["QTMEDIDAORCA"];
                if($item["IDMEDIDAFIXA"] == 'S' || $item["IDMULTIPVARIACAO"] == 'N'){
                    //number_format($item["QTMEDIDAORCA"],2, ",", ".") != number_format($item["QTMEDIDASERV"],2, ",", ".") ||
                    if(number_format($item["VRUNITITEMORCATBLPRECO"],2, ",", ".") != number_format($item["VRPRECO"],2, ",", ".")){
                        $this->atualizarValorItemProposta($nrseqorcamento, $nrorg, $item["NRITEMORCAMENTO"], $item["CDSERVORCARELAC"], $item["VRUNITITEMORCA"], $item["QTMEDIDAORCA"], $item["IDTPDESCORCA"], $item["VRPERCDESCORCA"], $item["VRUNITITEMORCATBLPRECO"]);
                        $this->atualizarPrecoElementoItem($nrseqorcamento, $nrorg, $item);
                        $this->replicaAteracaoParaItemPai($nrseqorcamento, $nrorg, $cdoperador, $item["CDSERVORCARELAC"]);
                        $validacao = true;
                    }
                } else {
                    $vrunitario = $item["VRPRECO"] * $item["QTMEDIDASERV"];
                    $qtmedida   = $item["VRUNITITEMORCATBLPRECO"]/$item["VRPRECO"];
                    $vrpreco    = $item["VRUNITITEMORCATBLPRECO"]/$item["QTMEDIDASERV"];
                    if(number_format($vrunitario, 2, ",", ".") != number_format($item["VRUNITITEMORCATBLPRECO"], 2, ",", ".") || number_format($vrpreco,2, ",", ".") != number_format($item["VRPRECO"],2, ",", ".") || number_format($qtmedida,2, ",", ".") != number_format($item["QTMEDIDASERV"],2, ",", ".")){
                        $this->atualizarValorItemProposta($nrseqorcamento, $nrorg, $item["NRITEMORCAMENTO"], $item["CDSERVORCARELAC"], $item["VRUNITITEMORCA"], $item["QTMEDIDAORCA"], $item["IDTPDESCORCA"], $item["VRPERCDESCORCA"], $item["VRUNITITEMORCATBLPRECO"]);
                        $this->atualizarPrecoElementoItem($nrseqorcamento, $nrorg, $item);
                        $this->replicaAteracaoParaItemPai($nrseqorcamento, $nrorg, $cdoperador, $item["CDSERVORCARELAC"]);
                        $validacao = true;
                    }
                }
            }
        }
        return $validacao;
    }
    /**
     * Atualiza o nome contratual dos itens
    **/
    public function salvarNomeContratual($rows, $nrorg, $cdoperador) {
        foreach($rows as $row){
            $params = array(
                "NRSEQORCAMENTO"       => $row["NRSEQORCAMENTO"],
                "IDELEMENTO"           => $row["IDELEMENTO"],
                "NMCONTRATUALITEMORCA" => $row["NMCONTRATUALITEMORCA"],
                "NRORG"                => $nrorg,
                "CDOPERADOR"           => $cdoperador
            );
            $this->itensFactory->salvarNomeContratual($params);
        }
    }

    public function atualizarValorItensNegociacaoValores($nrseqorcamento, $nritemorcamento, $vrtotalitemorca, $nritemorcamentopai, $nrorg, $cdoperador) {
        $params = array(
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        $dataset = $this->propostaFactory->getCodigoNegociacaoDoItem($params);
        $nrnegociacaoproposta = $dataset[0]["NRNEGOCIACAOPROPOSTA"];
        $params = array(
            "VRTOTITEM"            => $vrtotalitemorca,
            "CDOPERULTATU"         => $cdoperador,
            "NRORGULTATU"          => $nrorg,
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRITEMORCAMENTO"      => $nritemorcamento,
            "NRORG"                => $nrorg
        );
        $this->propostaFactory->atualizarValorItemNegociacao($params);

        //1- retorna a soma do valor dos elementos filhos do item
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRITEMORCAMENTOPAI"   => $nritemorcamentopai,
            "NRORG"                => $nrorg
        );
        $dataset = $this->propostaFactory->getSomaValorElementosItemNegociacao($params);
        $somaValoresItem = (float)$dataset[0]["VRTOTAL"];

        //2- atualiza o item pai com a soma dos filhos
        $params = array(
            "VRTOTITEM"            => $somaValoresItem,
            "CDOPERULTATU"         => $cdoperador,
            "NRORGULTATU"          => $nrorg,
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRITEMORCAMENTO"      => $nritemorcamentopai,
            "NRORG"                => $nrorg
        );
        $this->propostaFactory->atualizarValorItemNegociacao($params);
        
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $negociacoes = $this->propostaFactory->getNegociacoesProposta($params);

        foreach ($negociacoes as $nego) 
            $valor = $this->propostaService->atualizarValorNegociacaoComSomaDosItens($nego["NRNEGOCIACAOPROPOSTA"], $nrorg, $cdoperador);
            
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        $this->propostaFactory->excluirParcelasNegociacao($params);
    }

    /**
     * Valida se as composições de serviço da proposta estao ativas
    **/
    public function validarAtivacaoComposicao($nrseqorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $qtCompInativa = $this->itensFactory->getTotalComposicoesInativas($params);
        if($qtCompInativa > 0)
            throw new \Exception("POSSUI COMPOSICAO INATIVA");
    }

    public function verificarSeExisteDescontoNegociacaoProposta($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        
        $dataset = $this->itensFactory->verificarSeExisteDescontoNegociacaoProposta($params);
        return ((float)$dataset[0]["VRPERCDESCITEM"] > 0);
    }
    
    public function verificarSeExisteDescontoItensNegociacaoProposta($nrnegociacaoproposta, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );

        $dataset = $this->itensFactory->getSomaPercentualDescontoItensNegociacaoProposta($params);
        return ((float)$dataset[0]["VRPERCDESCITEM"] > 0);
    }

    public function getServicoPercentualMinimoDescontoPermitidoNegociacaoProposta($nrnegociacaoproposta, $cdoperador, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        $dataset = $this->itensFactory->getServicosNegociacaoProposta($params);

        $servicos = array();
        foreach ($dataset as $row) {
            $servicos[] = $row["CDSERVORCA"];
        }

        $servicos = implode(",", $servicos);
        
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        );
        $dataset = $this->descontoAcrescimoFactory->getServicosComPercentualDescontoPermitido($params, $servicos);
        
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    public function getCodigoNegociacaoItemProposta($nritemorcamento, $nrorg) {
        $params = array(
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        $dataset = $this->itensFactory->getCodigoNegociacaoItemProposta($params);
        return (count($dataset) > 0) ? $dataset[0]["NRNEGOCIACAOPROPOSTA"] : null;
    }

    public function getCodigoItemPropostaPai($nritemorcamento, $nrorg) {
        $params = array(
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        $dataset = $this->itensFactory->getCodigoItemPropostaPai($params);
        return $dataset[0]["NRITEMORCAMENTOPAI"];
    }

    public function getParametrosFaturamentoParaItemNegociacaoProposta($cdservorca, $nrseqorcamento, $nrtipoorcamento, $nrorg) {
        $params = array(
            "CDSERVORCA"      => $cdservorca,
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        $dataset = $this->itensFactory->getParametrosFaturamentoParaItemNegociacaoProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    /**
     * Insere um item de negociação de valores na proposta com um item de proposta informado. 
     * Se já existir uma negociação de valores com os mesmos parâmetros de faturamento do item, 
     * o item é inserido nela. Se não existir, é criada uma nova negociação para o item.
     */
    public function inserirItemNegociacaoValoresProposta($row, $nrorg, $cdoperador) {
        $this->entityManager->beginTransaction();
        try {
            $inseriuNegociacao = false;
            //retorna os parâmetros de faturamento do escopo ou do item do escopo, filtrando o serviço inserido.
            $paramsFatItem = $this->getParametrosFaturamentoParaItemNegociacaoProposta($row["CDSERVORCA"], $row["NRSEQORCAMENTO"], $row["NRTIPOORCAMENTO"], $nrorg);
            
            if ($paramsFatItem) {
                if ($paramsFatItem["CDFILIALFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Unidade para Faturamento" não foi encontrado para o item "' . $row["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDTIPOOPERFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Tipo de Operação da Nota Fiscal" não foi encontrado para o item "' . $row["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDSERIENF"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Série da Nota Fiscal" não foi encontrado para o item "' . $row["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
                }
                if ($paramsFatItem["CDPRODFATURA"] == null) {
                    throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Produto para Faturamento" não foi encontrado para o item "' . $row["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo ou do Serviço.');
                }

                // Busca se já existe uma negociação inserida na proposta com os mesmos parâmetros de faturamento do item.
                // Se já existe, o item será inserido nela.
                // Se não existe, insere uma nova negociação para depois inserir o item.
                $nrnegociacaoproposta = $this->getNegociacaoPropostaPorParametrosFaturamento($row["NRSEQORCAMENTO"], $nrorg, $row["NRTIPOORCAMENTO"], $paramsFatItem["IDTPFATURA"], $paramsFatItem["CDFILIALFATURA"], $paramsFatItem["NRPARCFATURA"], $paramsFatItem["CDTIPOOPERFATURA"], $paramsFatItem["CDSERIENF"], $paramsFatItem["CDEMPRESA"], $paramsFatItem["CDINSCESTA"], $paramsFatItem["CDINSCMUNI"]);
                
                if (is_null($nrnegociacaoproposta)) {
                    $nrnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_NEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
                    $params = array(
                        "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
                        "NRSEQORCAMENTO"       => $row["NRSEQORCAMENTO"],
                        "NRTIPOORCAMENTO"      => $row["NRTIPOORCAMENTO"],
                        "IDTPFATURA"           => $paramsFatItem["IDTPFATURA"],
                        "CDFILIALFATURA"       => $paramsFatItem["CDFILIALFATURA"],
                        "NRPARCFATURA"         => $paramsFatItem["NRPARCFATURA"],
                        "CDTIPOOPERFATURA"     => $paramsFatItem["CDTIPOOPERFATURA"],
                        "CDSERIENF"            => $paramsFatItem["CDSERIENF"],
                        "CDEMPRESA"            => $paramsFatItem["CDEMPRESA"],
                        "CDINSCESTA"           => $paramsFatItem["CDINSCESTA"],
                        "CDINSCMUNI"           => $paramsFatItem["CDINSCMUNI"],
                        "VRTOTITEM"            => $row["VRTOTALITEMORCA"],
                        "VRPERCDESCITEM"       => null,
                        "VRDESCITEM"           => null,
                        "VRTOTITEMCOMDESC"     => $row["VRTOTALITEMORCA"],
                        "IDATIVO"              => "S",
                        "NRORG"                => $nrorg,
                        "NRORGINCLUSAO"        => $nrorg,
                        "CDOPERINCLUSAO"       => $cdoperador
                    );
                    $this->propostaFactory->inserirNegociacaoProposta($params);
                    $inseriuNegociacao = true;
    
                    // Se o tipo de faturamento da negociação não for parcelamento serão inseridas as competências na negociação,
                    // tendo como base a parametrização do escopo de faturamento ou do seu item.
                    if ($paramsFatItem["IDTPFATURA"] != "P") {
                        $params = array(
                            "NRTIPOORCAMENTO" => $row["NRTIPOORCAMENTO"],
                            "CDSERVORCA"      => $row["CDSERVORCA"],
                            "NRORG"           => $nrorg
                        );
                        $competencia = $this->propostaFactory->getCompetenciaItemNegociacao($params);
                        
                        foreach ($competencia as $comp) {
                            $nrcompetencianegoprop = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOPROP", $nrorg, 1, 10, true);
                            $params = array(
                                "NRCOMPETENCIANEGOPROP" => $nrcompetencianegoprop,
                                "NRNEGOCIACAOPROPOSTA"  => $nrnegociacaoproposta,
                                "NRDIACOMPETINI"        => $comp["NRDIACOMPETINI"],
                                "NRDIACOMPETFIN"        => $comp["NRDIACOMPETFIN"],
                                "IDMESCOMPETINI"        => $comp["IDMESCOMPETINI"],
                                "IDMESCOMPETFIN"        => $comp["IDMESCOMPETFIN"],
                                "QTDIASAPURACAO"        => $comp["QTDIASAPURACAO"],
                                "IDTPCALCVENC"          => $comp["IDTPCALCVENC"],
                                "NRDIACALCVENC"         => $comp["NRDIACALCVENC"],
                                "IDMESCALCVENC"         => $comp["IDMESCALCVENC"],
                                "QTDIASFATUVENC"        => $comp["QTDIASFATUVENC"],
                                "IDATIVO"               => "S",
                                "NRORG"                 => $nrorg,
                                "NRORGINCLUSAO"         => $nrorg,
                                "CDOPERINCLUSAO"        => $cdoperador
                            );
                            $this->propostaFactory->inserirCompetenciaNegociacaoProposta($params);
                        }
                    }
                }
    
                // Insere o item na negociação de valores da proposta
                $nritnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_ITNEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
                $params = array(
                    "NRITNEGOCIACAOPROPOSTA" => $nritnegociacaoproposta,
                    "NRNEGOCIACAOPROPOSTA"   => $nrnegociacaoproposta,
                    "NRITEMORCAMENTO"        => $row["NRITEMORCAMENTO"],
                    "NRITEMORCAMENTOPAI"     => $row["NRITEMORCAMENTOPAI"],
                    "VRTOTITEM"              => $row["VRTOTALITEMORCA"],
                    "VRPERCDESCITEM"         => null,
                    "VRDESCITEM"             => null,
                    "VRTOTITEMCOMDESC"       => $row["VRTOTALITEMORCA"],
                    "CDSERVORCA"             => $row["CDSERVORCA"],
                    "CDPRODFATURA"           => $paramsFatItem["CDPRODFATURA"],
                    "NRPARCFATURA"           => $paramsFatItem["NRPARCFATURA"],
                    "IDATIVO"                => "S",
                    "NRORG"                  => $nrorg,
                    "NRORGINCLUSAO"          => $nrorg,
                    "CDOPERINCLUSAO"         => $cdoperador
                );
                $this->propostaFactory->inserirItemNegociacaoProposta($params);
    
                // Se não inseriu uma negociação nova, seu valor é atualizado com a soma dos itens 
                // e os valores da aba "Desconto por Período" também são atualizados.
                if (!$inseriuNegociacao) {
                    $valor = $this->propostaService->atualizarValorNegociacaoComSomaDosItens($nrnegociacaoproposta, $nrorg, $cdoperador);
    
                    //atualiza os registros da aba de Desconto por Periodo
                    $this->propostaService->atualizarValoresDescontoPorPeriodoDaNegociacao($valor, $nrnegociacaoproposta, $nrorg, $cdoperador);

                    //atualiza as parcelas da negociação com o valor a mais do item que foi inserido
                    $this->propostaService->reajustarParcelasNegociacao($row["NRSEQORCAMENTO"], $nrnegociacaoproposta, $nrorg, $cdoperador);
                }
            }

            $valorProposta = $this->propostaService->atualizarValorProposta($row["NRSEQORCAMENTO"], $nrorg);

            $this->entityManager->commit();

            return $valorProposta;
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function getNegociacaoPropostaPorParametrosFaturamento($nrseqorcamento, $nrorg, $nrtipoorcamento, $idtpfatura, $cdfilialfatura, $nrparcfatura, $cdtipooperfatura, $cdserienf, $cdempresa, $cdinscesta, $cdinscmuni) {
        $params = array(
            "NRSEQORCAMENTO"   => $nrseqorcamento,
            "NRORG"            => $nrorg,
            "NRTIPOORCAMENTO"  => $nrtipoorcamento,
            "IDTPFATURA"       => $idtpfatura,
            "CDFILIALFATURA"   => $cdfilialfatura,
            "NRPARCFATURA"     => $nrparcfatura,
            "CDTIPOOPERFATURA" => $cdtipooperfatura,
            "CDSERIENF"        => $cdserienf,
            "CDEMPRESA"        => $cdempresa,
            "CDINSCESTA"       => $cdinscesta,
            "CDINSCMUNI"       => $cdinscmuni
        );
        
        $dataset = $this->itensFactory->getNegociacaoPropostaPorParametrosFaturamento($params);
        return (count($dataset) > 0) ? $dataset[0]["NRNEGOCIACAOPROPOSTA"] : null;
    }

    public function excluirItemProposta($nritemorcamento, $nrorg) {
        $params = array(
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        $this->itensFactory->excluirItemProposta($params);
    }
    
    /**
     * retorna o valor unítario do serviço na tabela de preço
    **/
    public function getValorUnitarioTblPrecoItemProposta($nrtipoorcamento, $cdservorca, $cdmedidavar, $qtdmedida, $nrorg) {
        $params = array(
            "CDTPMEDIDAVARIAVEL" => $cdmedidavar,
            "QTMEDIDASERV"       => $qtdmedida,
            "NRTIPOORCAMENTO"    => $nrtipoorcamento,
            "CDSERVORCA"         => $cdservorca,
            "NRORG"              => $nrorg
        );
        
        $vrpreco = $this->itensFactory->getValorUnitarioTblPrecoItemProposta($params);
        if(count($vrpreco) > 0)
            return $vrpreco[0]["VRPRECO"];
        
        return null;
    }
    
    /**
     * Atualiza o valor dos itens na tabela de rateio
    **/
    public function atualizarValorRateioItens($nrseqorcamento, $nritemorcamento, $nrnegociacaoproposta, $nrorg, $cdoperador) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRITEMORCAMENTO"      => $nritemorcamento,
            "NRORG"                => $nrorg
        );
        $itens = $this->itensFactory->getValorItemNegociacaoDeValores($params);
        
        foreach($itens as $item) {
            $valorTotalItem = $item["VRTOTITEMCOMDESC"];
            $params = array(
                "NRSEQORCAMENTO"   => $nrseqorcamento,
                "NRITEMORCAMENTO"  => $nritemorcamento,
                "NRORG"            => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "VRTOTALITEM"      => $valorTotalItem
            );
            $this->itensFactory->atualizarValorRateioItens($params);
        }
    }

    public function getMedidaVariavelEscopo($nrtipoorcamento, $cdservorca, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "CDSERVORCA"      => $cdservorca,
            "NRORG"           => $nrorg
        );

        $dataset = $this->itensFactory->getMedidaVariavelEscopo($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }

    public function excluirMedidaVariavelServico($nrseqorcamento, $cdservorca, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "CDSERVORCA"     => $cdservorca,
            "NRORG"          => $nrorg
        );

        $this->itensFactory->excluirMedidaVariavelServico($params);
    }

    public function DeletaItemOrcamentoConcluido($nrseqorcamento, $nritemorcamento, $nrtipoorcamento, $cdservorca, $nrorg, $cdoperador) {
        $this->entityManager->beginTransaction();
        try {
            $this->excluiItemAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg);
            
            $idGeraRateio = $this->itFormacaoService->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
            if(count($idGeraRateio) > 0 && $idGeraRateio[0]["IDGERARATEIO"] == 'S')
                $this->itFormacaoService->geraRateios(array(), $nrseqorcamento, false, null, $nrtipoorcamento);
            
            $this->excluirMedidaVariavelServico($nrseqorcamento, $cdservorca, $nrorg);
            $this->entityManager->commit();
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function excluirMedidasVariaveisPropostaSemItemRelacionado($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->itensFactory->getMedidasVariaveisPropostaSemItemRelacionado($params);
        if (count($dataset) > 0) {
            $this->excluirMedidaVariavelServico($nrseqorcamento, $dataset[0]["CDSERVORCA"], $nrorg);
        }
    }
}
?>