<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\Sm00004108RenovacaoContratoFactory;
use Teknisa\SM\Factory\Sm00004003ReajusteContratoFactory;
use Teknisa\SM\Factory\Sm00004000ContratoFactory;
use Teknisa\SM\Factory\IntegracaoOmFactory;
use Teknisa\SM\Service\Sm00004003ReajusteContratoService;
use Teknisa\SM\Service\IntegracoesTeknisaService;
use Teknisa\SM\Service\IntegracaoOmService;
use Teknisa\SM\Service\Sm00004000ContratoService;
use Teknisa\SM\Util\Procedure\FctGerarParcelasFatura;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Session\Session;

class Sm00004108RenovacaoContratoService {
    protected $renovacaofactory;
    protected $entityManager;
    protected $connectioncustom;
    protected $connection;
    protected $reajusteService;
    
    public function __construct(Sm00004108RenovacaoContratoFactory $renovacaofactory, EntityManager $entityManager, ConnectionCustom $connectioncustom, Sm00004003ReajusteContratoService $reajusteService, NewCode $newCode) {
        $this->renovacaofactory = $renovacaofactory;
        $this->entityManager    = $entityManager;
        $this->connectioncustom = $connectioncustom;
        $this->reajusteService  = $reajusteService;
        $this->connection       = $entityManager->getConnection();
        $this->newCode          = $newCode;
    }
    
    /**
     * Valida a conexão de uma determinada organização do SM
    **/
    public function getConnectionEntityManagerOrganizacaoSm($nrorg) {
        $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);
        if($entityManagerConnection !== false)
            $this->entityManager = $entityManagerConnection;
        
        $this->connection = $this->entityManager->getConnection();
    }
    
    /**
     * Retorna todas as organizações que utilizam o SM
    **/
    public function getOrganizacoesSM() {
        $this->getConnectionEntityManagerOrganizacaoSm(20);
        $this->renovacaofactory = new Sm00004108RenovacaoContratoFactory($this->entityManager);
        
        return $this->renovacaofactory->getOrganizacoesSM();
    }
    
    /**
     * Valida a conexão e solicita a renovação automática dos contratos
    **/
    public function renovarContratosAutomaticamente($nrorg) {
        $this->getConnectionEntityManagerOrganizacaoSm($nrorg);
        $this->newCode          = new NewCode($this->connection, "NOVO_CODIGO_SC");
        $this->renovacaofactory = new Sm00004108RenovacaoContratoFactory($this->entityManager);
        
        $reajusteContratoFactory = new Sm00004003ReajusteContratoFactory($this->entityManager);
        $contratoFactory = new Sm00004000ContratoFactory($this->entityManager);
        $integracaoOmFactory = new IntegracaoOmFactory($this->entityManager);
        
        $session = new Session();
        $integracoesTeknisaService = new IntegracoesTeknisaService($session);
        $environment = new Environment($session);
        
        $integracaoOmService = new IntegracaoOmService($integracaoOmFactory, $environment, $session, $this->connectioncustom, $this->entityManager, $integracoesTeknisaService, null, null, null);
        
        $contratoService = new Sm00004000ContratoService($this->entityManager, $contratoFactory, $environment, null, null, null, null, $this->newCode, $integracaoOmService, null, $session, null);
        
        $this->reajusteService = new Sm00004003ReajusteContratoService($this->entityManager, $this->newCode, $reajusteContratoFactory, $contratoService, $contratoFactory, $integracaoOmService);
        
        return $this->renovarContrato(array(), $nrorg, true);
    }
    
    /**
     * Realiza a renovação de um contrato.
     * 
     * @params{array} $nrseqcontrato - números dos contratos a ser reajustados.
     * Obs: Quando não enviado valor, a função faz a renovação automática dos contratos
    **/
    public function renovarContrato(array $contratos = array(), $nrorg, $idrenovautomatica, $dataIniVigencia = null, $nrdiasProrrog = null, $idreajustar = null, $cdoperador = null) {
        try {
            if ($idrenovautomatica) {
                $contratos = $this->renovacaofactory->getContratosVencidosComRenovacaoAutomatica($nrorg);
            }
            
            foreach($contratos as $contrato) {
                $this->connection->beginTransaction();
                $mensagemerror = null;
                
                $codContrato = isset($contrato["NRCONTRATOFCT"]) ? ($contrato["NRCONTRATOFCT"] . " (" . $contrato["NRSEQCONTRATO"] . ")") : $contrato["NRSEQCONTRATO"];
                
                try {
                    if ($idrenovautomatica) {
                        $cdoperador    = $contrato["CDOPERULTATU"];
                        $idreajustar   = $contrato["IDREAJUSTEAUTOMATICO"] == "S";
                        $nrdiasProrrog = $contrato["NRDIASPRORGCONTRENOV"];
                        
                        $dataIniVigencia = DateUtil::getDataDeString($contrato["DTFINVIGENCONT"]);
                        $dataIniVigencia = DateUtil::adicionaIntervalo($dataIniVigencia, 1)->format("d/m/Y");
                    }
                    
                    $params = array(
                        "NRSEQCONTRATO"   => $contrato["NRSEQCONTRATO"],
                        "NRDIASPRORGCONT" => $nrdiasProrrog,
                        "DTINIVIGENCONT"  => $dataIniVigencia,
                        "NRORG"           => $nrorg
                    );

                    if($idreajustar) {
                        $bloqueiaReajuste = $this->reajusteService->validaBloqReajusteContrIndividual($nrorg, $contrato["NRSEQCONTRATO"], $contrato["NRTIPOORCAMENTO"]);
                        if ($bloqueiaReajuste == "BLOQUEAR") {
                            throw new \Exception("Operação bloqueada!<br><br>Não será possível realizar o reajuste do contrato pois ele possui índices financeiros com o cadastro de alíquotas incompleto. Esses cadastros devem ser regularizados antes da realização dos reajustes dos contratos.");
                        }
                        else {
                            $result = $this->realizarReajusteContrato($contrato, $nrorg, $cdoperador);
                            if($result)
                                throw new \Exception($result);
                        }
                    }
                    
                    $this->renovacaofactory->alterarPeriodoVigenciaContrato($params);
                    $this->salvarProrrogContrato($contrato["NRSEQCONTRATO"], $dataIniVigencia, $nrdiasProrrog, $nrorg, $cdoperador, $idrenovautomatica);
                    
                    $procedure = new FctGerarParcelasFatura($this->connection);
                    $procedure->executar($contrato["NRSEQCONTRATO"], $nrorg, $cdoperador, 1);
                    
                    $this->reajusteService->refazerIntegracaoOM($contrato["NRSEQCONTRATO"], $nrorg);
                    
                    $this->connection->commit();
                    
                    if ($idrenovautomatica) {
                        $msgReajuste = $idreajustar ? "SIM" : "NÃO";
                        $mensagemLog = "Período de vigência atualizado. Reajuste feito = " . $msgReajuste;
                        
                        echo "ORGANIZAÇÃO: " . $nrorg . " | Contrato " . $codContrato . " | SUCESSO: " . $mensagemLog . "\n";
                        
                        try {
                            $params = array (
                                "NRSEQCONTRATO"  => $contrato["NRSEQCONTRATO"],
                                "DTFINVIGENCONT" => $contrato["DTFINVIGENCONT"],
                                "DTINIVIGENCONT" => $contrato["DTINIVIGENCONT"],
                                "VRCONTRATO"     => $contrato["VRCONTRATO"],
                                "ERROR"          => null
                            );
                            
                            $this->renovacaofactory->inserirLogRenovacaoContrato($params);
                        }
                        catch(\Exception $e) {
                            echo "ORGANIZAÇÃO: " . $nrorg . " - Falha ao inserir log de sucesso do contrato " . $contrato["NRSEQCONTRATO"] . ": \n" . $e->getMessage();
                        }
                    }
                }
                catch (\Exception $e) {
                    $this->connection->rollback();
                    if (!$idrenovautomatica) {
                        throw $e;
                    }
                    else {
                        $mensagemerror = $e->getMessage();
                        echo "ORGANIZAÇÃO: " . $nrorg . " | Contrato " . $codContrato . " | ERRO: " . $mensagemerror . "\n";
                        
                        try {
                            $params = array (
                                "NRSEQCONTRATO"  => $contrato["NRSEQCONTRATO"],
                                "DTFINVIGENCONT" => $contrato["DTFINVIGENCONT"],
                                "DTINIVIGENCONT" => $contrato["DTINIVIGENCONT"],
                                "VRCONTRATO"     => $contrato["VRCONTRATO"],
                                "ERROR"          => $mensagemerror
                            );
                            
                            $this->renovacaofactory->inserirLogRenovacaoContrato($params);
                        }
                        catch(\Exception $e) {
                            echo "ORGANIZAÇÃO: " . $nrorg . " - Falha ao inserir log de erro do contrato " . $contrato["NRSEQCONTRATO"] . ": \n" . $e->getMessage();
                        }
                    }
                }
            }
            
            return $contratos;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Realiza o reajuste de um contrato
    **/
    public function realizarReajusteContrato($contrato, $nrorg, $cdoperador) {
        $params = array(
            "NRSEQCONTRATO" => "_" . $contrato["NRSEQCONTRATO"] . "_",
            "NRORG"         => $nrorg
        );
        
        $paramsReajuste = $this->renovacaofactory->getParametrosReajusteContrato($params);
        if(count($paramsReajuste) == 0) {
            return "Parâmetros de reajuste não encontratos.";
        }
        else {
            $somaPercentual = 0.0;
            foreach ($paramsReajuste as $param) {
                $somaPercentual += (float)$param["NRPERCENTINDICE"];
            }

            if ($somaPercentual != 100) {
                return "A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste deste contrato.";
            }

            $this->reajusteService->reajusteContrMassa($paramsReajuste, $cdoperador, true);
        }
        
        return null;
    }
    
    /**
     * Busca na tabela de parametrização geral se o sistema permite o envio de relatorios de renovação
    **/
    public function getParametrosGeracaoRelatorio($nrorg) {
        $paramgeral = $this->renovacaofactory->getParametrosGeracaoRelatorio($nrorg);
        return (count($paramgeral) == 1) ? $paramgeral[0] : null;
    }

    /**
     * deleta da tabela temporaria os logs e contratos que deram error
    **/
    public function deletarLogErroRenovacao($nrorg) {
        $params = array("NRORG" => $nrorg);
        $this->renovacaofactory->deletarLogErroRenovacao($params);
    }
    
    /**
     * Busca todos os contratos vencidos e que não possuem renovação automática
    **/
    public function getContratosSemRenovacaoAutomatica($nrorg, $tipoPeriodo) {
        $params = array(
            "PERIODO" => $tipoPeriodo,
            "NRORG"   => $nrorg
        );
        $dataset = $this->renovacaofactory->getContratosSemRenovacaoAutomatica($params);
        return (int)$dataset[0]["QTD"];
    }

    public function salvarProrrogContrato($nrseqcontrato, $dataIniVigencia, $nrdiasProrrog, $nrorg, $cdoperador, $idrenovautomatica) {
        $msgRenovacaoAuto = $idrenovautomatica ? "automática" : "manual";
        $dsprorrcont = "Inserido pela rotina $msgRenovacaoAuto de Renovação de Contratos.";
        
        $dataFinalVigencia = DateUtil::getDataDeString($dataIniVigencia);
        $dataFinalVigencia = DateUtil::adicionaIntervalo($dataFinalVigencia, $nrdiasProrrog-1)->format("d/m/Y");
        
        $nrseqprorrcont = $this->newCode->getCodeByTableName("NRSEQPRORRCONT", $nrorg, 1, 10, true);
        
        $params = array(
            "NRSEQPRORRCONT"  => $nrseqprorrcont,
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "IDPRORRCONT"     => "P",
            "DTNVDATAINICIAL" => $dataIniVigencia,
            "DTNVDATAFINAL"   => $dataFinalVigencia,
            "DSPRORRCONT"     => $dsprorrcont,
            "IDDTATIVO"       => "S",
            "NRORG"           => $nrorg,
            "NRORGINCLUSAO"   => $nrorg,
            "CDOPERINCLUSAO"  => $cdoperador,
            "IDATIVO"         => "S"
        );
        
        $this->renovacaofactory->salvarProrrogContrato($params);
    }
    
    /**
     * Altera os parâmetros "IDRENOVACAOAUTOMATICA" e "IDREAJUSTEAUTOMATICO" para os contratos selecionados
    **/
    public function salvarRenovReajAuto($nrseqcontrato, $idrenovacaoautomatica, $idreajusteautomatico, $nrorg, $cdoperador) {
       
        $contratos = explode('_', $nrseqcontrato);
        
        foreach($contratos as $contrato){
        
            $params = array(
                "IDRENOVACAOAUTOMATICA" => $idrenovacaoautomatica,
                "IDREAJUSTEAUTOMATICO"  => $idreajusteautomatico,
                "NRORG"                 => $nrorg,
                "OPERADOR"               => $cdoperador,
                "NRSEQCONTRATO"         => $contrato
            );
            
            $this->renovacaofactory->salvarRenovReajAuto($params);
        }
        
    }
    
    /**
     * Renova e reajusta os contratos selecionados
    **/
    public function renovContrMassa($contratos, $nrorg, $cdoperador) {
        try {
            $this->connection->beginTransaction();
            
            foreach($contratos as $contrato) {
                
                /* Reajustar Contrato */ 
                if ($contrato["IDREAJUSTEAUTOMATICO"] == 'Y') {

                    $contr = array("NRSEQCONTRATO" => $contrato["NRSEQCONTRATO"]);
                    $result = $this->realizarReajusteContrato($contr, $nrorg, $cdoperador);
                    if($result) {
                        throw new \Exception($result);
                    }
                
                    unset($contr);
                }
                
                $dtIniRenovacao = explode(" ", $contrato["DTINIRENOVACAO"])[0];
                
                $params = array(
                    "NRSEQCONTRATO"   => $contrato["NRSEQCONTRATO"],
                    "NRDIASPRORGCONT" => $contrato["NRDIASRENOVACAO"],
                    "DTINIVIGENCONT"  => $dtIniRenovacao,
                    "NRORG"           => $nrorg
                );
                
                $this->renovacaofactory->alterarPeriodoVigenciaContrato($params);
                $this->salvarProrrogContrato($contrato["NRSEQCONTRATO"], $dtIniRenovacao, $contrato["NRDIASRENOVACAO"], $nrorg, $cdoperador, false);
                
                /*Regera as parcelas de Faturamento do contrato*/
                $procedure = new FctGerarParcelasFatura($this->connection);
                $procedure->executar($contrato["NRSEQCONTRATO"], $nrorg, $cdoperador, 1);
                
                /*Refaz a integração com o OM*/    
                $this->reajusteService->refazerIntegracaoOM($contrato["NRSEQCONTRATO"], $nrorg);
                
            }  
            
            $this->connection->commit();
        
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
    
     /**
     * Recebe os dados de Indice Financeiro
    **/
    public function getDadosIndFinanc($nrseqcontrato, $nrorg) {
    
        $params = array(
            "NRSEQCONTRATO"   => '_'.$nrseqcontrato.'_',
            "NRORG"           => $nrorg
        );
        
        $dados = $this->renovacaofactory->getDadosIndFinanc($params);
        
        return $dados;
    }
    
}

?>