/* Sistemas Integrados TEKNISA
   Compras e Suprimentos

Nome: 8640_SUPSTPO.sql

Descrio:
Atualiza o preo do produto da lista de preo na CUSTOSUP considerando
particularidades e produto preferencial.

Criao:
02/12/2003 Adriano de Arajo
29/10/2004 Converso para SQLSERVER - Marcelo Alexsandro
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'LISPREC_UPPRODFILI'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
   DROP PROCEDURE LISPREC_UPPRODFILI;
GO

CREATE PROCEDURE LISPREC_UPPRODFILI
(
   @P_CDFILIAL     VARCHAR(4),
   @P_NRSEQPRELIST VARCHAR(10),
   @P_ATUCUSTO     VARCHAR(1) = 'N',
   @P_CDFILCUSTO   VARCHAR(4) = NULL,
   @P_CDLISTAREG   VARCHAR(255) = NULL
)
AS
  DECLARE @V_COUNT DOUBLE PRECISION;
  DECLARE @V_VRPRECULISM1 DOUBLE PRECISION;
  DECLARE @V_VRPRECULISM2 DOUBLE PRECISION;

  DECLARE @V_POSMINPRECO INT;
  DECLARE @V_POSMINPREPI INT;
  DECLARE @V_MINPRECO DOUBLE PRECISION;

  DECLARE @V_NRCOLEPRECO VARCHAR(10);
  DECLARE @V_CDREGIAO VARCHAR(3);
  DECLARE @V_CDFORNECED VARCHAR(21);
  DECLARE @V_CDPRODUTO VARCHAR(10);
  DECLARE @V_CDMARCA VARCHAR(5);
  DECLARE @V_IDTIPOAPUR VARCHAR(1);

  DECLARE @V_MULTIPLO INT;

  DECLARE @C_CDFILIAL     VARCHAR(4);
  DECLARE @C_CDPRODUTO    VARCHAR(10);
  DECLARE @C_DTCUSTOSUP   DATETIME;
  DECLARE @C_NRCOLETA     VARCHAR(10);
  DECLARE @C_VRPRCUSTOAF  DOUBLE PRECISION;

  DECLARE @ACORDO_CDFILIAL VARCHAR(4)
  DECLARE @ACORDO_DTINIVALITM DATETIME;
  DECLARE @ACORDO_DTFINVALITM DATETIME;
  DECLARE @ACORDO_CDPRODESTO VARCHAR(10)
  DECLARE @ACORDO_MINPRECO VARCHAR(200);

  DECLARE @CUSTO_CDFILIAL VARCHAR(4)
  DECLARE @CUSTO_DTINIVALITM DATETIME;
  DECLARE @CUSTO_DTFINVALITM DATETIME;
  DECLARE @CUSTO_CDPRODESTO VARCHAR(10)
  DECLARE @CUSTO_MINPRECO VARCHAR(200);
  
  DECLARE @CUSTO_VRMINPRECO DOUBLE PRECISION;
  DECLARE @CUSTO_VRPRECOPI DOUBLE PRECISION;
  
  DECLARE @V_IDUTCUSTOM1  VARCHAR(1);
  DECLARE @V_IDUTCUSTOM2  VARCHAR(1);

  DECLARE @V_DTINIVALLIST DATETIME;

  DECLARE @V_COUNT_AUX    DOUBLE PRECISION;

  DECLARE @V_CUSLISTINFLAC_DTCUSTOSUP DATETIME;
  DECLARE @V_CUSLISTINFLAC_DTFINVALITM DATETIME;

  DECLARE @V_CUSTOPOST_CDFILIAL     VARCHAR(4);
  DECLARE @V_CUSTOPOST_CDPRODUTO    VARCHAR(10);
  DECLARE @V_CUSTOPOST_DTCUSTOSUP   DATETIME;
  DECLARE @V_CUSTOPOST_VRCUSPRLISTP DOUBLE PRECISION;
  DECLARE @V_CUSTOPOST_CDFILCOTLIST VARCHAR(4);
  DECLARE @V_CUSTOPOST_NRCOLEPRLIST VARCHAR(10);
  DECLARE @V_CUSTOPOST_CDREGIAOLIST VARCHAR(3);
  DECLARE @V_CUSTOPOST_CDFORNECLIST VARCHAR(21);
  DECLARE @V_CUSTOPOST_CDPRODLIST   VARCHAR(10);
  DECLARE @V_CUSTOPOST_CDMARCALIST  VARCHAR(5);
  DECLARE @V_CUSTOPOST_VRCUSPRLISM1 DOUBLE PRECISION;
  DECLARE @V_CUSTOPOST_VRCUSPRLISM2 DOUBLE PRECISION;
  DECLARE @V_CUSTOPOST_VRHISPRLISTP DOUBLE PRECISION;
  DECLARE @V_CUSTOPOST_DTFINVALLIST DATETIME;
  DECLARE @C_VERIF_CUSTO_CDFILIAL VARCHAR(4);
  DECLARE @C_VERIF_CUSTO_CDPRODUTO VARCHAR(10);
  DECLARE @C_VERIF_CUSTO_DTCUSTOSUP DATETIME;
  DECLARE @V_IDUTCUSLIPR  VARCHAR(1);
  DECLARE @V_VRFATOCONV   FLOAT;
  DECLARE @V_VRIMPITM     FLOAT;

  DECLARE @V_CDIMPOSTRIB  VARCHAR(2);
  DECLARE @V_IDVRIMPSTRIB VARCHAR(1);
  DECLARE @C_CDPRODLIST   VARCHAR(10);

  DECLARE @V_IMPOSTO_CDIMPOSTO    VARCHAR(2);
  DECLARE @V_IMPOSTO_IDTRATIMPO   VARCHAR(1);
  DECLARE @V_IMPOSTO_IDIMPORECUP  VARCHAR(1);
  DECLARE @V_IMPOSTO_CDCCTBLATIV  VARCHAR(20);
  DECLARE @V_IMPOSTO_CDCCTBLPASS  VARCHAR(20);
  DECLARE @V_IMPOSTO_VRADIMPCOL   FLOAT;
  DECLARE @V_IMPOSTO_VRALIQIMPCOL FLOAT;
  DECLARE @V_IMPOSTO_VRIMPCOLETA  FLOAT;
  DECLARE @V_IMPOSTO_CDFISCITMCOL VARCHAR(1);

  DECLARE @V_REGIAO_CDREGIAO      VARCHAR(3);
  DECLARE @V_FILIAL_CDFILIAL      VARCHAR(4);

  DECLARE @V_ULTIMAREGIAO         VARCHAR(3);
  DECLARE @V_ULTIMAFILIAL         VARCHAR(4);
  DECLARE @V_ULTIMOPRODUTO        VARCHAR(10);

  DECLARE @V_QTFILIALLISPREC      DOUBLE PRECISION;
  DECLARE @V_QTREGIAOLISPREC      DOUBLE PRECISION;
  DECLARE @V_QTPRODLISPREC        DOUBLE PRECISION;

  DECLARE @V_AUX_REGIAOGERAL      DOUBLE PRECISION;
  DECLARE @V_AUX_FILIAL           DOUBLE PRECISION;
  DECLARE @V_AUX_CUSTO            DOUBLE PRECISION;

  DECLARE @V_LISTATEMPORARIA      VARCHAR(10);

  DECLARE @V_VRPERAJFORNFIL       DOUBLE PRECISION;
  DECLARE @V_IDCONSPERPIAJPR      VARCHAR(1);
  DECLARE @V_PIDSV                DOUBLE PRECISION;

  DECLARE @V_DTFINVALLIST         DATETIME;

BEGIN
  /*
    Essa procedure tem a funo de calcular o custo da lista de preos e
     separada em dois processos. O primeiro processo l os dados referentes
    a lista de preos passada como parmetro. Esse processo grava os dados
    em uma tabela temporria, e apartir dessa tabela temporria so feitas
    as consistncias e verificaes dos cadastros necessrios. O segundo processo
    l todos os acordos existentes que ainda esto vigentes, ou seja, com a data
    de validade final maior que a data atual, e grava em outra tabela temporria,
    na qual so efetuadas as mesmas consistncias.
  */

  SET @V_MULTIPLO = 1000000;
  SET @V_VRFATOCONV = 1;
  SET @V_VRIMPITM = 0;
  SELECT
      @V_IDUTCUSTOM1 = ISNULL(MAX(IDUTTPCUSTO), 'N')
   FROM
      TIPOCUSTO
   WHERE
      CDTIPOCUSTO = '30';

   SELECT
      @V_IDUTCUSTOM2 = ISNULL(MAX(IDUTTPCUSTO), 'N')
   FROM
      TIPOCUSTO
   WHERE
      CDTIPOCUSTO = '31';

  SELECT
    @V_IDTIPOAPUR = MAX(IDTIPOAPUR)
  FROM
    PARASUPRI
  WHERE
    CDFILIAL = @P_CDFILIAL;

  SELECT
    @V_DTINIVALLIST = MAX(DTINIVALLIST),
    @V_DTFINVALLIST = MAX(DTFINVALLIST)
  FROM PREVLISTA
  WHERE
    CDFILIAL     = @P_CDFILIAL AND
    NRSEQPRELIST = @P_NRSEQPRELIST;

   SELECT
      @V_IDUTCUSLIPR = ISNULL(MAX(IDUTTPCUSTO), 'N')
   FROM
      TIPOCUSTO
   WHERE
      CDTIPOCUSTO = '10';

  IF (@V_IDUTCUSLIPR = 'N') BEGIN
     SELECT
        @V_IDUTCUSLIPR = ISNULL(MAX(IDUTTPCUSTO), 'N')
     FROM
        TIPOCUSTO
     WHERE
        CDTIPOCUSTO = '48';
  END;


  DECLARE C_VERIF_CUSTO CURSOR FOR
       SELECT
          CUSTOSUP.CDFILIAL,
          CUSTOSUP.CDPRODUTO,
          CUSTOSUP.DTCUSTOSUP
       FROM
          CUSTOSUP,
          COLETAPR,
          ITEMCOLE,
          PREVLISTA
       WHERE
          PREVLISTA.CDFILIAL      = @P_CDFILIAL             AND
          PREVLISTA.NRSEQPRELIST  = @P_NRSEQPRELIST         AND
          PREVLISTA.DTINIVALLIST <= ITEMCOLE.DTINIVALITM   AND
          PREVLISTA.DTFINVALLIST >= ITEMCOLE.DTFINVALITM   AND
          ITEMCOLE.CDFILIAL       = PREVLISTA.CDFILIAL     AND
          ITEMCOLE.CDFILIAL       = COLETAPR.CDFILIAL      AND
          ITEMCOLE.NRCOLEPRECO    = COLETAPR.NRCOLEPRECO   AND
          COLETAPR.CDOPERADOR     = PREVLISTA.CDOPERADOR   AND
          ITEMCOLE.CDFILIAL       = CUSTOSUP.CDFILCOTLIST  AND
          ITEMCOLE.NRCOLEPRECO    = CUSTOSUP.NRCOLEPRLIST  AND
          ITEMCOLE.CDMARCA        = CUSTOSUP.CDMARCALIST   AND
          ITEMCOLE.CDREGIAO       = CUSTOSUP.CDREGIAOLIST  AND
          ITEMCOLE.CDFORNECED     = CUSTOSUP.CDFORNECLIST  AND
          ITEMCOLE.CDPRODUTO      = CUSTOSUP.CDPRODLIST    AND
          CUSTOSUP.VRPRCUSTOAF IS NULL                    AND
          (EXISTS(SELECT
                     '1'
                  FROM
                     FORNECEDOR
                  WHERE
                     FORNECEDOR.CDFORNECED = CUSTOSUP.CDFORNECLIST AND
                     ISNULL(FORNECEDOR.IDSITUCADA, 'A') <> 'A'
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     FORNECEDOR
                  WHERE
                     FORNECEDOR.CDFORNECED = CUSTOSUP.CDFORNECLIST AND
                     ISNULL(FORNECEDOR.IDTIPOFORN, 'N') <> 'N'
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     REFORFIL
                  WHERE
                     (REFORFIL.CDFILIAL   = CUSTOSUP.CDFILIAL)     AND
                     (REFORFIL.CDFORNECED = CUSTOSUP.CDFORNECLIST) AND
                     (REFORFIL.CDPERIENTR IS NULL)
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     REFOREG
                  WHERE
                    REFOREG.CDREGIAO   = CUSTOSUP.CDREGIAOLIST AND
                    REFOREG.CDFORNECED = CUSTOSUP.CDFORNECLIST
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QFOPRODFORN,
                     PARASUPRI
                  WHERE
                     (PARASUPRI.CDFILIAL     = @P_CDFILIAL)            AND
                     (QFOPRODFORN.CDPRODUTO  = CUSTOSUP.CDPRODLIST)    AND
                     (QFOPRODFORN.CDFORNECED = CUSTOSUP.CDFORNECLIST) AND
                     ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN) - DAY(DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN))) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN))))) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QFOMARPROD,
                     PARASUPRI
                  WHERE
                     (PARASUPRI.CDFILIAL   = @P_CDFILIAL)           AND
                     (QFOMARPROD.CDPRODUTO = CUSTOSUP.CDPRODLIST)   AND
                     (QFOMARPROD.CDMARCA   = CUSTOSUP.CDMARCALIST) AND
                     ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR ((QFOMARPROD.IDAPRMARPRO = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA) - DAY(DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA))) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > QFOMARPROD.DTVALAPROVA))))) OR
           NOT EXISTS(SELECT
                        '1'
                     FROM
                        QFOMARPROD
                     WHERE
                        (QFOMARPROD.CDPRODUTO = CUSTOSUP.CDPRODLIST) AND
                        (QFOMARPROD.CDMARCA   = CUSTOSUP.CDMARCALIST)

                     UNION

                     SELECT
                        '1'
                     FROM
                        PARASUPRI
                     WHERE
                        (PARASUPRI.CDFILIAL    = @P_CDFILIAL) AND
                        (PARASUPRI.IDMARSEMQUA = 'N' OR PARASUPRI.IDMARSEMQUA IS NULL)
                      ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QUALIFICACAO,
                     FORNECEDOR,
                     PARASUPRI
                  WHERE
                     (FORNECEDOR.CDFORNECED     = CUSTOSUP.CDFORNECLIST) AND
                     (QUALIFICACAO.CDQUALIF     = FORNECEDOR.CDQUALIF)   AND
                     (PARASUPRI.CDFILIAL        = @P_CDFILIAL)            AND
                     (FORNECEDOR.IDFORNAUTO     = 'N')                   AND
                     ((QUALIFICACAO.IDPERCOTCMP = '2') OR ((PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > FORNECEDOR.DTVALQUALIF) OR (PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF) - DAY(DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF)))))) OR
           ((NOT EXISTS (SELECT
                           '1'
                        FROM
                           PARTFILI
                        WHERE
                           (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL)      AND
                           (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST)     AND
                           (PARTFILI.CDTIPPART IN ('1', '2'))            AND
                           (PARTFILI.CDMARCA   = CUSTOSUP.CDMARCALIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL   = CUSTOSUP.CDFILIAL)   AND
                                (PARTFILI.CDPRODUTO  = CUSTOSUP.CDPRODLIST)  AND
                                (PARTFILI.CDTIPPART  IN ('4', '5'))         AND
                                (PARTFILI.CDFORNECED = CUSTOSUP.CDFORNECLIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL   = CUSTOSUP.CDFILIAL)      AND
                                (PARTFILI.CDPRODUTO  = CUSTOSUP.CDPRODLIST)     AND
                                (PARTFILI.CDTIPPART  IN ('7', '8'))            AND
                                (PARTFILI.CDMARCA    = CUSTOSUP.CDMARCALIST)   AND
                                (PARTFILI.CDFORNECED = CUSTOSUP.CDFORNECLIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL)   AND
                                (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST)  AND
                                (PARTFILI.CDTIPPART  IN ('3','6','9'))))

            AND (EXISTS (SELECT
                            '1'
                         FROM
                            PARTFILI
                         WHERE
                            (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL) AND
                            (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST)))
           ) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL  = CUSTOSUP.CDFILIAL)     AND
                      (CDPRODUTO = CUSTOSUP.CDPRODLIST)    AND
                      (CDTIPPART = '3')                   AND
                      (CDMARCA   = CUSTOSUP.CDMARCALIST)) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL   = CUSTOSUP.CDFILIAL)  AND
                      (CDPRODUTO  = CUSTOSUP.CDPRODLIST) AND
                      (CDTIPPART  = '6')                AND
                      (CDFORNECED = CUSTOSUP.CDFORNECLIST)) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL   = CUSTOSUP.CDFILIAL)      AND
                      (CDPRODUTO  = CUSTOSUP.CDPRODLIST)     AND
                      (CDTIPPART  = '9')                    AND
                      (CDMARCA    = CUSTOSUP.CDMARCALIST)   AND
                      (CDFORNECED = CUSTOSUP.CDFORNECLIST))
           )
  OPEN C_VERIF_CUSTO;

  FETCH NEXT FROM C_VERIF_CUSTO INTO @C_VERIF_CUSTO_CDFILIAL, @C_VERIF_CUSTO_CDPRODUTO, @C_VERIF_CUSTO_DTCUSTOSUP;

  WHILE (@@FETCH_STATUS = 0)  BEGIN
      DELETE FROM CUSTOSUP
      WHERE
        @C_VERIF_CUSTO_CDFILIAL   = CUSTOSUP.CDFILIALORIG AND
        @C_VERIF_CUSTO_CDPRODUTO  = CUSTOSUP.CDPRODORIG   AND
        @C_VERIF_CUSTO_DTCUSTOSUP = CUSTOSUP.DTCUSTSUPORI;

      DELETE FROM CUSTOSUP
      WHERE
         CUSTOSUP.CDFILIAL   = @C_VERIF_CUSTO_CDFILIAL   AND
         CUSTOSUP.CDPRODUTO  = @C_VERIF_CUSTO_CDPRODUTO  AND
         CUSTOSUP.DTCUSTOSUP = @C_VERIF_CUSTO_DTCUSTOSUP;
      FETCH NEXT FROM C_VERIF_CUSTO INTO @C_VERIF_CUSTO_CDFILIAL, @C_VERIF_CUSTO_CDPRODUTO, @C_VERIF_CUSTO_DTCUSTOSUP;
   END;
   CLOSE C_VERIF_CUSTO;
   DEALLOCATE C_VERIF_CUSTO;


  /* Cursor: C_DELCUSTACOR

     Tratamento dos custos de lista de preos da qual a cotao de acordo
     no mais existe. Os campos da cotao de acordo sero limpos e o valor
     do custo ser substitudo pelo valor da lista(atravs do campo VRHISPRLISTP),
     se no houver os campos da lista e nem custo de AF, o registro ser excludo.
  */
  DECLARE C_DELCUSTACOR CURSOR FOR
    SELECT
       CUSTOSUP.CDFILIAL,
       CUSTOSUP.CDPRODUTO,
       CUSTOSUP.DTCUSTOSUP,
       CUSTOSUP.NRCOLEPRLIST,
       CUSTOSUP.VRPRCUSTOAF,
       CUSTOSUP.CDMARCALIST,
       CUSTOSUP.CDREGIAOLIST,
       CUSTOSUP.CDFORNECLIST,
       CUSTOSUP.CDPRODLIST
    FROM
       CUSTOSUP
    WHERE
       (CUSTOSUP.CDFILIAL   = ISNULL(@P_CDFILCUSTO, CUSTOSUP.CDFILIAL)) AND
       (CUSTOSUP.DTCUSTOSUP >= @V_DTINIVALLIST) AND
       (CUSTOSUP.IDCUSLISTINF <> 'S') AND
       NOT EXISTS
         (SELECT
             NRCOLEPRECO
          FROM
             ITEMCOLE
          WHERE
             (ITEMCOLE.CDFILIAL    = CUSTOSUP.CDFILCOTACOR) AND
             (ITEMCOLE.NRCOLEPRECO = CUSTOSUP.NRCOLEPRACOR) AND
             (ITEMCOLE.CDMARCA     = CUSTOSUP.CDMARCAACOR)  AND
             (ITEMCOLE.CDREGIAO    = CUSTOSUP.CDREGIAOACOR) AND
             (ITEMCOLE.CDFORNECED  = CUSTOSUP.CDFORNECACOR) AND
             (ITEMCOLE.CDPRODUTO   = CUSTOSUP.CDPRODACOR) );

  OPEN C_DELCUSTACOR;

  FETCH NEXT FROM C_DELCUSTACOR INTO @C_CDFILIAL, @C_CDPRODUTO, @C_DTCUSTOSUP, @C_NRCOLETA, @C_VRPRCUSTOAF, @V_CDMARCA, @V_CDREGIAO, @V_CDFORNECED, @C_CDPRODLIST;

  WHILE (@@FETCH_STATUS = 0)  BEGIN
     IF (@C_NRCOLETA IS NOT NULL) OR (@C_VRPRCUSTOAF IS NOT NULL) BEGIN
        IF @V_IDUTCUSLIPR = 'S' BEGIN
           SET @V_VRIMPITM = 0;
           DECLARE C_IMPOSTO CURSOR FOR
              SELECT
                 IMPOSTO.CDIMPOSTO,
                 IMPOSTO.IDTRATIMPO,
                 IMPOSTO.IDIMPORECUP,
                 IMPOSTO.CDCCTBLATIV,
                 IMPOSTO.CDCCTBLPASS,
                 IMPCOLET.VRADIMPCOL,
                 IMPCOLET.VRALIQIMPCOL,
                 IMPCOLET.VRIMPCOLETA,
                 IMPCOLET.CDFISCITMCOL
              FROM
                 IMPCOLET,
                 IMPOSTO
              WHERE
                 IMPCOLET.CDFILIAL    = @P_CDFILIAL     AND
                 IMPCOLET.NRCOLEPRECO = @C_NRCOLETA     AND
                 IMPCOLET.CDFORNECED  = @V_CDFORNECED   AND
                 IMPCOLET.CDREGIAO    = @V_CDREGIAO     AND
                 IMPCOLET.CDPRODUTO   = @C_CDPRODLIST   AND
                 IMPCOLET.CDMARCA     = @V_CDMARCA      AND
                 IMPCOLET.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
           OPEN C_IMPOSTO;
           FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                          @V_IMPOSTO_IDTRATIMPO,
                                          @V_IMPOSTO_IDIMPORECUP,
                                          @V_IMPOSTO_CDCCTBLATIV,
                                          @V_IMPOSTO_CDCCTBLPASS,
                                          @V_IMPOSTO_VRADIMPCOL,
                                          @V_IMPOSTO_VRALIQIMPCOL,
                                          @V_IMPOSTO_VRIMPCOLETA,
                                          @V_IMPOSTO_CDFISCITMCOL;
           WHILE (@@FETCH_STATUS = 0)  BEGIN
              SELECT
                 @V_CDIMPOSTRIB  = MAX(CDIMPOSTO),
                 @V_IDVRIMPSTRIB = MAX(IDVRIMPSTRIB)
              FROM
                 IMPOSTO
              WHERE
                 CDIMPOSTRIB = @V_IMPOSTO_CDIMPOSTO;

              IF (@V_IMPOSTO_IDIMPORECUP = 'S') BEGIN
                 IF (@V_IMPOSTO_CDFISCITMCOL = '1') BEGIN
                    IF (@V_IMPOSTO_CDCCTBLATIV IS NOT NULL) BEGIN
                       IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                          SET @V_VRIMPITM = @V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                       END;
                    END;
                 END
                 ELSE BEGIN
                    IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                       IF @V_CDIMPOSTRIB IS NULL BEGIN
                          SET @V_VRIMPITM = @V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                       END;
                    END
                    ELSE BEGIN
                       IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                          SET @V_VRIMPITM = @V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                       END;
                    END;
                 END;
              END
              ELSE BEGIN
                 IF @V_IMPOSTO_CDCCTBLPASS IS NOT NULL BEGIN
                    IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                       SET @V_VRIMPITM = @V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                    END;
                 END
                 ELSE BEGIN
                    IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                       IF @V_CDIMPOSTRIB IS NULL BEGIN
                          SET @V_VRIMPITM = @V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                       END;
                    END
                    ELSE BEGIN
                       IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                          SET @V_VRIMPITM = @V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                       END;
                    END;
                 END;
              END;
              FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                             @V_IMPOSTO_IDTRATIMPO,
                                             @V_IMPOSTO_IDIMPORECUP,
                                             @V_IMPOSTO_CDCCTBLATIV,
                                             @V_IMPOSTO_CDCCTBLPASS,
                                             @V_IMPOSTO_VRADIMPCOL,
                                             @V_IMPOSTO_VRALIQIMPCOL,
                                             @V_IMPOSTO_VRIMPCOLETA,
                                             @V_IMPOSTO_CDFISCITMCOL;
           END;
           CLOSE C_IMPOSTO;
           DEALLOCATE C_IMPOSTO;
           SELECT
              @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
           FROM
              PRODUTO
           WHERE
              CDPRODUTO = @C_CDPRODLIST;
        END;

        UPDATE CUSTOSUP
        SET
          VRCUSPRLISTP = CASE @C_NRCOLETA
                          WHEN NULL THEN NULL
                          ELSE CUSTOSUP.VRHISPRLISTP
                         END,
          VRCUSPRLISM1 = NULL,
          VRCUSPRLISM2 = NULL,
          CDFILCOTACOR = NULL,
          NRCOLEPRACOR = NULL,
          CDMARCAACOR  = NULL,
          CDREGIAOACOR = NULL,
          CDFORNECACOR = NULL,
          CDPRODACOR   = NULL,
          VRCUSLIPRIMP = CASE @V_IDUTCUSLIPR
                         WHEN 'N' THEN NULL
                                  ELSE
                                  CASE @C_NRCOLETA
                                  WHEN NULL THEN NULL
                                            ELSE
                                           ((CUSTOSUP.VRHISPRLISTP*@V_VRFATOCONV) + @V_VRIMPITM)/@V_VRFATOCONV
                                  END
                         END
        WHERE
           (CDFILIAL   = @C_CDFILIAL)  AND
           (CDPRODUTO  = @C_CDPRODUTO) AND
           (DTCUSTOSUP = @C_DTCUSTOSUP);

     END
     ELSE BEGIN

        DELETE CUSTOSUP
        WHERE
           (CDFILIAL   = @C_CDFILIAL)  AND
           (CDPRODUTO  = @C_CDPRODUTO) AND
           (DTCUSTOSUP = @C_DTCUSTOSUP);

     END;

     FETCH NEXT FROM C_DELCUSTACOR INTO @C_CDFILIAL, @C_CDPRODUTO, @C_DTCUSTOSUP, @C_NRCOLETA, @C_VRPRCUSTOAF, @V_CDMARCA, @V_CDREGIAO, @V_CDFORNECED, @C_CDPRODLIST;
  END;
  CLOSE C_DELCUSTACOR;
  DEALLOCATE C_DELCUSTACOR;

  /* Cursor: C_DELCUSTLIST

      Tratamento dos custos de lista de preos da qual a cotao de lista
      no mais existe. Os campos da cotao de lista sero limpos, mas se os campos
      de cotao de acordo e de custo de af no estiverem preenchidos, o registro
      ser excludo.
  */
  DECLARE C_DELCUSTLIST CURSOR FOR
    SELECT
       CUSTOSUP.CDFILIAL,
       CUSTOSUP.CDPRODUTO,
       CUSTOSUP.DTCUSTOSUP,
       CUSTOSUP.NRCOLEPRACOR,
       CUSTOSUP.VRPRCUSTOAF
    FROM
       CUSTOSUP
    WHERE
       (CUSTOSUP.CDFILIAL   = ISNULL(@P_CDFILCUSTO, CUSTOSUP.CDFILIAL)) AND
       (CUSTOSUP.DTCUSTOSUP >= @V_DTINIVALLIST) AND
       (CUSTOSUP.IDCUSLISTINF <> 'S') AND
       NOT EXISTS
         (SELECT
             NRCOLEPRECO
          FROM
             ITEMCOLE
          WHERE
             (ITEMCOLE.CDFILIAL    = CUSTOSUP.CDFILCOTLIST) AND
             (ITEMCOLE.NRCOLEPRECO = CUSTOSUP.NRCOLEPRLIST) AND
             (ITEMCOLE.CDMARCA     = CUSTOSUP.CDMARCALIST) AND
             (ITEMCOLE.CDREGIAO    = CUSTOSUP.CDREGIAOLIST) AND
             (ITEMCOLE.CDFORNECED  = CUSTOSUP.CDFORNECLIST) AND
             (ITEMCOLE.CDPRODUTO   = CUSTOSUP.CDPRODLIST) )

  OPEN C_DELCUSTLIST;

  FETCH NEXT FROM C_DELCUSTLIST INTO @C_CDFILIAL, @C_CDPRODUTO, @C_DTCUSTOSUP, @C_NRCOLETA, @C_VRPRCUSTOAF;

  WHILE (@@FETCH_STATUS = 0)  BEGIN

     IF (@C_NRCOLETA IS NOT NULL) OR (@C_VRPRCUSTOAF IS NOT NULL) BEGIN
        UPDATE CUSTOSUP
        SET
          VRHISPRLISTP = NULL,
          CDFILCOTLIST = NULL,
          NRCOLEPRLIST = NULL,
          CDMARCALIST  = NULL,
          CDREGIAOLIST = NULL,
          CDFORNECLIST = NULL,
          CDPRODLIST   = NULL
        WHERE
           (CDFILIAL   = @C_CDFILIAL)  AND
           (CDPRODUTO  = @C_CDPRODUTO) AND
           (DTCUSTOSUP = @C_DTCUSTOSUP);

     END
     ELSE BEGIN

        DELETE CUSTOSUP
        WHERE
           (CDFILIAL   = @C_CDFILIAL)  AND
           (CDPRODUTO  = @C_CDPRODUTO) AND
           (DTCUSTOSUP = @C_DTCUSTOSUP);

     END;

     FETCH NEXT FROM C_DELCUSTLIST INTO @C_CDFILIAL, @C_CDPRODUTO, @C_DTCUSTOSUP, @C_NRCOLETA, @C_VRPRCUSTOAF;
  END;
  CLOSE C_DELCUSTLIST;
  DEALLOCATE C_DELCUSTLIST;

  IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
     SELECT
        @V_LISTATEMPORARIA = MAX(NRSEQULTLISPREC),
        @V_ULTIMAREGIAO = MAX(CDREGULTLISPREC),
        @V_ULTIMAFILIAL = MAX(CDFILULTLISPREC),
        @V_ULTIMOPRODUTO = MAX(CDPRODULLISPREC)
     FROM TEMPULTATULISPR
     WHERE CDCENULTLISPREC = @P_CDFILIAL
       AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
  END
  ELSE BEGIN
     SET @V_LISTATEMPORARIA = NULL;
     SET @V_ULTIMAREGIAO = NULL;
     SET @V_ULTIMAFILIAL = NULL;
     SET @V_ULTIMOPRODUTO = NULL;
  END;


  DELETE CUSTOSUP_TMP
  WHERE
    CDFILCENTCMP = @P_CDFILIAL AND
    NRSEQPRELIST = @P_NRSEQPRELIST;

  DELETE CUSTOSUPACOR_TMP
  WHERE
    CDFILCENTCMP = @P_CDFILIAL AND
    NRSEQPRELIST = @P_NRSEQPRELIST;

  IF (@P_CDLISTAREG IS NULL) BEGIN
     INSERT INTO CUSTOSUP_TMP
      (CDFILIAL,
       CDFILCENTCMP,
       NRSEQPRELIST,
       DTINIVALITM,
       DTFINVALITM,
       CDPRODESTO,
       NRCOLEPRECO,
       CDREGIAO,
       CDFORNECED,
       CDPRODUTO,
       CDMARCA,
       VRPRECOUN,
       VRPRECECO,
       VRPRECOPI)
     SELECT /*+ INDEX (ITEMCOLE CMP00289_FK) */
       FILIAL.CDFILIAL,
       @P_CDFILIAL,
       @P_NRSEQPRELIST,
       CASE CONVERT(DOUBLE PRECISION, @V_DTINIVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTINIVALITM) WHEN ABS(CONVERT(DOUBLE PRECISION, @V_DTINIVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTINIVALITM)) THEN @V_DTINIVALLIST ELSE ITEMCOLE.DTINIVALITM END,
       CASE CONVERT(DOUBLE PRECISION, @V_DTFINVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTFINVALITM) WHEN ABS(CONVERT(DOUBLE PRECISION, @V_DTFINVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTFINVALITM)) THEN ITEMCOLE.DTFINVALITM ELSE @V_DTFINVALLIST END,
       PRODUTO.CDPRODESTO,
       ITEMCOLE.NRCOLEPRECO,
       ITEMCOLE.CDREGIAO,
       ITEMCOLE.CDFORNECED,
       ITEMCOLE.CDPRODUTO,
       ITEMCOLE.CDMARCA,
       ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV AS VRPRECOUN,
       ITEMCOLE.VRPRECECOITM/PRODUTO.VRFATOCONV AS VRPRECECO,
       ITEMCOLE.VRPRECOITMPI/PRODUTO.VRFATOCONV AS VRPRECOPI
     FROM
       FILIAL,
       PRODUTO,
       ITEMCOLE
     WHERE
       ITEMCOLE.CDFILIAL        = @P_CDFILIAL          AND
       ITEMCOLE.NRSEQPRELIST    = @P_NRSEQPRELIST      AND
       ITEMCOLE.IDITMLISPREC    = 'S'                  AND
       ITEMCOLE.VRPRECUNIITM    > 0                    AND
       ISNULL(FILIAL.IDCNTRRESC, 'N') = 'N'            AND
       FILIAL.CDFILIAL          = ISNULL(@P_CDFILCUSTO, FILIAL.CDFILIAL) AND
       ((FILIAL.CDFILIAL        < @V_ULTIMAFILIAL) OR (@V_ULTIMAFILIAL IS NULL)) AND
       ((FILIAL.CDREGIAO        < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL)) AND
       ((ITEMCOLE.CDREGIAO      < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL)) AND
       ITEMCOLE.CDREGIAO        = FILIAL.CDREGIAO      AND
       ITEMCOLE.CDPRODUTO       = PRODUTO.CDPRODUTO;
  END
  ELSE BEGIN
     INSERT INTO CUSTOSUP_TMP
      (CDFILIAL,
       CDFILCENTCMP,
       NRSEQPRELIST,
       DTINIVALITM,
       DTFINVALITM,
       CDPRODESTO,
       NRCOLEPRECO,
       CDREGIAO,
       CDFORNECED,
       CDPRODUTO,
       CDMARCA,
       VRPRECOUN,
       VRPRECECO,
       VRPRECOPI)
     SELECT /*+ INDEX (ITEMCOLE CMP00289_FK) */
       FILIAL.CDFILIAL,
       @P_CDFILIAL,
       @P_NRSEQPRELIST,
       CASE CONVERT(DOUBLE PRECISION, @V_DTINIVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTINIVALITM) WHEN ABS(CONVERT(DOUBLE PRECISION, @V_DTINIVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTINIVALITM)) THEN @V_DTINIVALLIST ELSE ITEMCOLE.DTINIVALITM END,
       CASE CONVERT(DOUBLE PRECISION, @V_DTFINVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTFINVALITM) WHEN ABS(CONVERT(DOUBLE PRECISION, @V_DTFINVALLIST) - CONVERT(DOUBLE PRECISION, ITEMCOLE.DTFINVALITM)) THEN ITEMCOLE.DTFINVALITM ELSE @V_DTFINVALLIST END,
       PRODUTO.CDPRODESTO,
       ITEMCOLE.NRCOLEPRECO,
       ITEMCOLE.CDREGIAO,
       ITEMCOLE.CDFORNECED,
       ITEMCOLE.CDPRODUTO,
       ITEMCOLE.CDMARCA,
       ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV AS VRPRECOUN,
       ITEMCOLE.VRPRECECOITM/PRODUTO.VRFATOCONV AS VRPRECECO,
       ITEMCOLE.VRPRECOITMPI/PRODUTO.VRFATOCONV AS VRPRECOPI
     FROM
       FILIAL,
       PRODUTO,
       ITEMCOLE,
       LISTA LISTAREG
     WHERE
       ITEMCOLE.CDFILIAL        = @P_CDFILIAL          AND
       ITEMCOLE.NRSEQPRELIST    = @P_NRSEQPRELIST      AND
       ITEMCOLE.IDITMLISPREC    = 'S'                  AND
       ITEMCOLE.VRPRECUNIITM    > 0                    AND
       ISNULL(FILIAL.IDCNTRRESC, 'N') = 'N'            AND
       FILIAL.CDFILIAL          = ISNULL(@P_CDFILCUSTO, FILIAL.CDFILIAL) AND
       ((FILIAL.CDFILIAL        < @V_ULTIMAFILIAL) OR (@V_ULTIMAFILIAL IS NULL)) AND
       ((FILIAL.CDREGIAO        < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL)) AND
       ((ITEMCOLE.CDREGIAO      < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL)) AND
       LISTAREG.NRLISTA         = @P_CDLISTAREG        AND
       LISTAREG.CDLISTA         = ITEMCOLE.CDREGIAO    AND
       ITEMCOLE.CDREGIAO        = FILIAL.CDREGIAO      AND
       ITEMCOLE.CDPRODUTO       = PRODUTO.CDPRODUTO;
  END;
  IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
     IF @V_LISTATEMPORARIA IS NULL BEGIN
        INSERT INTO TEMPULTATULISPR (CDCENULTLISPREC,NRSEQULTLISPREC)
        VALUES (@P_CDFILIAL, @P_NRSEQPRELIST);
     END;
  END;
  SET @V_QTREGIAOLISPREC = NULL;

  DECLARE C_REGIAO CURSOR FOR
      SELECT
        CDREGIAO
      FROM
        REGIAO
      WHERE ((REGIAO.CDREGIAO < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL))
        AND EXISTS (SELECT 1
                    FROM CUSTOSUP_TMP
                    WHERE CUSTOSUP_TMP.CDREGIAO = REGIAO.CDREGIAO
                      AND CUSTOSUP_TMP.CDFILCENTCMP = @P_CDFILIAL
                      AND CUSTOSUP_TMP.NRSEQPRELIST = @P_NRSEQPRELIST
                    )

      ORDER BY CDREGIAO DESC;
  OPEN C_REGIAO;
  FETCH NEXT FROM C_REGIAO INTO @V_REGIAO_CDREGIAO;
  WHILE (@@FETCH_STATUS = 0)
  BEGIN
     SET @V_QTFILIALLISPREC = NULL;
     IF @V_QTREGIAOLISPREC IS NULL BEGIN
        SELECT
           @V_AUX_REGIAOGERAL = COUNT(1)
        FROM
           REGIAO
        WHERE ((REGIAO.CDREGIAO < @V_ULTIMAREGIAO) OR (@V_ULTIMAREGIAO IS NULL))
          AND EXISTS (SELECT 1
                      FROM CUSTOSUP_TMP
                      WHERE CUSTOSUP_TMP.CDREGIAO = REGIAO.CDREGIAO
                        AND CUSTOSUP_TMP.CDFILCENTCMP = @P_CDFILIAL
                        AND CUSTOSUP_TMP.NRSEQPRELIST = @P_NRSEQPRELIST);

        SET @V_QTREGIAOLISPREC = @V_AUX_REGIAOGERAL;
        IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
           UPDATE TEMPULTATULISPR
           SET QTREGIAOLISPREC = @V_QTREGIAOLISPREC
           WHERE CDCENULTLISPREC = @P_CDFILIAL
             AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
        END;
     END;

     DECLARE C_FILIAL CURSOR FOR
         SELECT
           CDFILIAL
         FROM
           FILIAL
         WHERE FILIAL.CDREGIAO = @V_REGIAO_CDREGIAO
           AND ((FILIAL.CDFILIAL < @V_ULTIMAFILIAL) OR (@V_ULTIMAFILIAL IS NULL))
           AND EXISTS (SELECT 1
                       FROM CUSTOSUP_TMP
                       WHERE CUSTOSUP_TMP.CDFILIAL = FILIAL.CDFILIAL
                         AND CUSTOSUP_TMP.CDREGIAO = @V_REGIAO_CDREGIAO
                         AND CUSTOSUP_TMP.CDFILCENTCMP = @P_CDFILIAL
                         AND CUSTOSUP_TMP.NRSEQPRELIST = @P_NRSEQPRELIST
                       )
         ORDER BY CDFILIAL DESC;
     OPEN C_FILIAL;
     FETCH NEXT FROM C_FILIAL INTO @V_FILIAL_CDFILIAL;
     WHILE (@@FETCH_STATUS = 0)
     BEGIN
        SET @V_QTPRODLISPREC = NULL;
        IF @V_QTFILIALLISPREC IS NULL BEGIN
           SELECT
             @V_AUX_FILIAL = COUNT(1)
           FROM
             FILIAL
           WHERE FILIAL.CDREGIAO = @V_REGIAO_CDREGIAO
             AND ((FILIAL.CDFILIAL < @V_ULTIMAFILIAL) OR (@V_ULTIMAFILIAL IS NULL))
             AND EXISTS (SELECT 1
                         FROM CUSTOSUP_TMP
                         WHERE CUSTOSUP_TMP.CDFILIAL = FILIAL.CDFILIAL
                           AND CUSTOSUP_TMP.CDREGIAO = @V_REGIAO_CDREGIAO
                           AND CUSTOSUP_TMP.CDFILCENTCMP = @P_CDFILIAL
                           AND CUSTOSUP_TMP.NRSEQPRELIST = @P_NRSEQPRELIST
                         );

           SET @V_QTFILIALLISPREC = @V_AUX_FILIAL;
           IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
              UPDATE TEMPULTATULISPR
              SET QTFILIALLISPREC = @V_QTFILIALLISPREC
              WHERE CDCENULTLISPREC = @P_CDFILIAL
                AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
           END;
        END;

        SELECT
           @V_AUX_CUSTO = COUNT(DISTINCT CUSTOSUP_TMP.CDPRODESTO)
        FROM CUSTOSUP_TMP
        WHERE CUSTOSUP_TMP.CDFILIAL = @V_FILIAL_CDFILIAL
          AND CUSTOSUP_TMP.CDREGIAO = @V_REGIAO_CDREGIAO;

        DECLARE C_CUSTO CURSOR FOR
         (SELECT
            CUSTOSUP_TMP.CDFILIAL,
            CUSTOSUP_TMP.DTINIVALITM,
            CUSTOSUP_TMP.DTFINVALITM,
            CUSTOSUP_TMP.CDPRODESTO,
            MIN(CASE @V_IDTIPOAPUR
                 WHEN NULL THEN CUSTOSUP_TMP.VRPRECOUN
                 WHEN '1'  THEN CUSTOSUP_TMP.VRPRECECO
                 ELSE CUSTOSUP_TMP.VRPRECOPI
                END * @V_MULTIPLO) OVER(PARTITION BY CUSTOSUP_TMP.CDFILIAL, CUSTOSUP_TMP.DTINIVALITM, CUSTOSUP_TMP.DTFINVALITM, CUSTOSUP_TMP.CDPRODESTO) AS VRMINPRECO,
            CUSTOSUP_TMP.VRPRECOUN,
            CUSTOSUP_TMP.NRCOLEPRECO +
            CUSTOSUP_TMP.CDREGIAO    +
            CUSTOSUP_TMP.CDPRODUTO   +
            CUSTOSUP_TMP.CDMARCA     +
            CUSTOSUP_TMP.CDFORNECED
          FROM
            FORNECEDOR,
            COLETAPR,
            CUSTOSUP_TMP LEFT JOIN PRODPREFMED
                                ON CUSTOSUP_TMP.CDFILIAL   = PRODPREFMED.CDFILIAL  AND
                                   CUSTOSUP_TMP.CDPRODESTO = PRODPREFMED.CDPRODUTO

          WHERE
            @P_CDFILIAL              = COLETAPR.CDFILIAL   AND
            CUSTOSUP_TMP.NRCOLEPRECO = COLETAPR.NRCOLEPRECO AND
            CUSTOSUP_TMP.CDREGIAO    = @V_REGIAO_CDREGIAO   AND
            CUSTOSUP_TMP.CDFILIAL    = @V_FILIAL_CDFILIAL   AND
            CUSTOSUP_TMP.CDPRODUTO   = ISNULL(PRODPREFMED.CDPRODPREF, CUSTOSUP_TMP.CDPRODUTO) AND
            CUSTOSUP_TMP.CDFORNECED  = FORNECEDOR.CDFORNECED AND
            FORNECEDOR.IDSITUCADA    = 'A'                   AND
            ISNULL(FORNECEDOR.IDTIPOFORN, 'N') <> 'A'       AND
            NOT EXISTS
             (SELECT
                CDFILIAL
              FROM
                REFORFIL
              WHERE
                CDFILIAL   = CUSTOSUP_TMP.CDFILIAL   AND
                CDFORNECED = CUSTOSUP_TMP.CDFORNECED AND
                CDPERIENTR IS NULL) AND

            NOT EXISTS
             (SELECT
                REFOREG.CDREGIAO
              FROM
                REFOREG
              WHERE
                REFOREG.CDREGIAO   = CUSTOSUP_TMP.CDREGIAO AND
                REFOREG.CDFORNECED = CUSTOSUP_TMP.CDFORNECED) AND

            NOT EXISTS
             (SELECT
                 QFOPRODFORN.CDPRODUTO
              FROM
                 QFOPRODFORN,
                 PARASUPRI
              WHERE
                PARASUPRI.CDFILIAL = @P_CDFILIAL  AND
                (QFOPRODFORN.CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                (QFOPRODFORN.CDFORNECED= CUSTOSUP_TMP.CDFORNECED) AND
                ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR
                 ((QFOPRODFORN.IDFORNAPROV = 'S') AND
                    ((PARASUPRI.IDULTIDTVALQ = 'M' AND
                      COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN) - DAY(DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN))) OR
                     (PARASUPRI.IDULTIDTVALQ = 'D' AND
                      COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN)) ) )) AND

            NOT EXISTS
              (SELECT
                 QFOMARPROD.CDMARCA
               FROM
                 QFOMARPROD, PARASUPRI
               WHERE
                (PARASUPRI.CDFILIAL = @P_CDFILIAL)            AND
                (QFOMARPROD.CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                (QFOMARPROD.CDMARCA   = CUSTOSUP_TMP.CDMARCA)     AND
                ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR
                 ((QFOMARPROD.IDAPRMARPRO = 'S') AND
                    ((PARASUPRI.IDULTIDTVALQ = 'M' AND
                      COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA) - DAY(DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA))) OR
                     (PARASUPRI.IDULTIDTVALQ = 'D' AND
                      COLETAPR.DTCOLEPREC > QFOMARPROD.DTVALAPROVA)) ) )) AND

            EXISTS
              (SELECT
                 QFOMARPROD.CDMARCA
               FROM
                 QFOMARPROD
               WHERE
                 QFOMARPROD.CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO AND
                 QFOMARPROD.CDMARCA   = CUSTOSUP_TMP.CDMARCA

               UNION

               SELECT
                 PARASUPRI.CDMARCA
               FROM
                 PARASUPRI
               WHERE
                 PARASUPRI.CDFILIAL    = @P_CDFILIAL AND
                 (PARASUPRI.IDMARSEMQUA = 'N' OR PARASUPRI.IDMARSEMQUA IS NULL)) AND

            NOT EXISTS
              (SELECT QUALIFICACAO.CDQUALIF
               FROM QUALIFICACAO, PARASUPRI
               WHERE (FORNECEDOR.CDQUALIF       = QUALIFICACAO.CDQUALIF) AND
                     (PARASUPRI.CDFILIAL        = @P_CDFILIAL)           AND
                     (FORNECEDOR.IDFORNAUTO     = 'N')                   AND
                     ((QUALIFICACAO.IDPERCOTCMP = '2')                   OR
                      ((PARASUPRI.IDULTIDTVALQ  = 'D'                    AND
                        COLETAPR.DTCOLEPREC > FORNECEDOR.DTVALQUALIF)       OR
                      (PARASUPRI.IDULTIDTVALQ = 'M'                      AND
                       COLETAPR.DTCOLEPREC > DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF) - DAY(DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF)))))) AND

            (      (EXISTS (SELECT
                        1
                    FROM
                        PARTFILI
                    WHERE
                        (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)    AND
                        (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                        (CDTIPPART IN ('1', '2'))        AND
                        (CDMARCA   = CUSTOSUP_TMP.CDMARCA)   AND
                        (NOT EXISTS (SELECT
                                        1
                                     FROM
                                        PARTFILI
                                     WHERE
                                        (CDFILIAL   = CUSTOSUP_TMP.CDFILIAL ) AND
                                        (CDPRODUTO  = CUSTOSUP_TMP.CDPRODUTO) AND
                                        (CDTIPPART  IN ('4', '5'))            AND
                                        (NRCOLEPRECO IS NULL)                 AND
                                        (CDFORNECED <> CUSTOSUP_TMP.CDFORNECED))

                         )
                      )



                   )

                  OR (EXISTS (SELECT
                                 1
                              FROM
                                 PARTFILI
                              WHERE
                                 (CDFILIAL   = CUSTOSUP_TMP.CDFILIAL )    AND
                                 (CDPRODUTO  = CUSTOSUP_TMP.CDPRODUTO)  AND
                                 (CDTIPPART  IN ('4', '5'))         AND
                                 (NRCOLEPRECO IS NULL)              AND
                                 (CDFORNECED = CUSTOSUP_TMP.CDFORNECED) AND
                                 (NOT EXISTS (SELECT
                                                 1
                                              FROM
                                                 PARTFILI
                                              WHERE
                                                (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)    AND
                                                (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                                                (CDTIPPART IN ('1', '2'))        AND
                                                (CDMARCA   <> CUSTOSUP_TMP.CDMARCA))
                                   )
                                )

                      ) OR

             (EXISTS (SELECT
                         CDFORNECED
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                         (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                         (CDTIPPART  = '7')                   AND
                         (CDMARCA    = CUSTOSUP_TMP.CDMARCA)      AND
                         (CDFORNECED = CUSTOSUP_TMP.CDFORNECED))     ) OR
             (EXISTS (SELECT
                         CDFORNECED
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                         (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                         (CDTIPPART  = '8')                   AND
                         (CDMARCA    = CUSTOSUP_TMP.CDMARCA)      AND
                         (CDFORNECED = CUSTOSUP_TMP.CDFORNECED))     ) OR
             (EXISTS (SELECT
                         CDPRODUTO
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)  AND
                         (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                         (CDTIPPART  IN ('3','6','9')))       AND
                         NOT EXISTS (SELECT
                                        1
                                     FROM
                                        PARTFILI
                                     WHERE
                                        (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)    AND
                                        (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                                        (NRCOLEPRECO IS NULL)            AND
                                        (CDTIPPART  IN ('1','2','4','5','7','8')))



                          ) OR
             (NOT EXISTS (SELECT
                         CDPRODUTO
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                         (NRCOLEPRECO IS NULL)                         AND
                         (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO))    )
            ) AND
              (NOT EXISTS (SELECT
                              CDMARCA
                           FROM
                              PARTFILI
                           WHERE
                              (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                              (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                              (CDTIPPART = '3')                   AND
                              (CDMARCA   = CUSTOSUP_TMP.CDMARCA))         )   AND
              (NOT EXISTS (SELECT
                              CDFORNECED
                           FROM
                              PARTFILI
                           WHERE
                              (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                              (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                              (CDTIPPART  = '6')                   AND
                              (CDFORNECED = CUSTOSUP_TMP.CDFORNECED))     )   AND
              (NOT EXISTS (SELECT
                              CDFORNECED
                           FROM
                              PARTFILI
                           WHERE
                              (CDFILIAL  = CUSTOSUP_TMP.CDFILIAL)           AND
                              (CDPRODUTO = CUSTOSUP_TMP.CDPRODUTO) AND
                              (CDTIPPART  = '9')                   AND
                              (CDMARCA    = CUSTOSUP_TMP.CDMARCA)      AND
                              (CDFORNECED = CUSTOSUP_TMP.CDFORNECED))     ));
        OPEN C_CUSTO;
        FETCH NEXT FROM C_CUSTO INTO @CUSTO_CDFILIAL, @CUSTO_DTINIVALITM, @CUSTO_DTFINVALITM, @CUSTO_CDPRODESTO, @CUSTO_VRMINPRECO, @CUSTO_VRPRECOPI, @CUSTO_MINPRECO;
        WHILE (@@FETCH_STATUS = 0)
        BEGIN
          IF @V_QTPRODLISPREC IS NULL BEGIN
             SET @V_QTPRODLISPREC = @V_AUX_CUSTO;
             IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
                UPDATE TEMPULTATULISPR
                SET QTPRODLISPREC = @V_QTPRODLISPREC
                WHERE CDCENULTLISPREC = @P_CDFILIAL
                  AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
             END;
          END;

          SET @V_COUNT = 0;
          SELECT
             @V_COUNT = COUNT(*)
          FROM
             CUSTOSUP
          WHERE
             CDFILIAL   = @CUSTO_CDFILIAL   AND
             CDPRODUTO  = @CUSTO_CDPRODESTO AND
             DTCUSTOSUP = @CUSTO_DTINIVALITM;

         /* SET @V_POSMINPREPI = CHARINDEX('#', @CUSTO_MINPRECO);
          SET @V_POSMINPRECO = CHARINDEX('%', @CUSTO_MINPRECO);*/

          SET @V_MINPRECO    = @CUSTO_VRMINPRECO/@V_MULTIPLO; 
          SET @V_NRCOLEPRECO = SUBSTRING(@CUSTO_MINPRECO, 1, 10);
          SET @V_CDREGIAO    = SUBSTRING(@CUSTO_MINPRECO, 11, 3);
          SET @V_CDPRODUTO   = SUBSTRING(@CUSTO_MINPRECO, 14, 10);
          SET @V_CDMARCA     = SUBSTRING(@CUSTO_MINPRECO, 24, 5);
          SET @V_CDFORNECED  = SUBSTRING(@CUSTO_MINPRECO, 29, 21);

          SET @V_VRPERAJFORNFIL = NULL;
          SET @V_IDCONSPERPIAJPR = 'N';
          SELECT
             @V_VRPERAJFORNFIL = MAX(VRPERAJFORNFIL),
             @V_IDCONSPERPIAJPR = MAX(IDCONSPERPIAJPR)
          FROM AJUSTEPRFORNFIL
          WHERE AJUSTEPRFORNFIL.CDFORNECED = @V_CDFORNECED
            AND AJUSTEPRFORNFIL.CDFILIAL   = @CUSTO_CDFILIAL
            AND AJUSTEPRFORNFIL.DTINIAJPRFORFIL = (SELECT MAX(A.DTINIAJPRFORFIL)
                                                   FROM AJUSTEPRFORNFIL A
                                                   WHERE A.CDFORNECED = @V_CDFORNECED
                                                     AND A.CDFILIAL = @CUSTO_CDFILIAL

                                                     AND CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103)
                                                         BETWEEN A.DTINIAJPRFORFIL AND ISNULL(A.DTFINAJPRFORFIL, CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103)));

          SET @V_PIDSV = NULL;
          IF @V_IDCONSPERPIAJPR = 'S' BEGIN
             SELECT
                @V_PIDSV = MAX(VRPERPIPRFO)
             FROM PRODFORNCOMP
             WHERE PRODFORNCOMP.CDFORNECED  = @V_CDFORNECED
               AND PRODFORNCOMP.CDPRODCOMPL = @V_CDPRODUTO;

             IF @V_PIDSV IS NULL BEGIN
                SELECT @V_PIDSV = MAX(VRPERPIFORN)
                FROM FORNECEDOR
                WHERE FORNECEDOR.CDFORNECED = @V_CDFORNECED;
             END;
          END;

          IF @V_PIDSV IS NULL BEGIN
             SET @V_PIDSV = 0;
          END;
          IF @V_VRPERAJFORNFIL IS NULL BEGIN
             SET @V_VRPERAJFORNFIL = 0;
          END;

          SET @V_MINPRECO = @V_MINPRECO - (@V_MINPRECO * @V_PIDSV/100) + (@V_MINPRECO * @V_VRPERAJFORNFIL/100);

          IF @V_COUNT = 0 BEGIN
            SET @V_VRPRECULISM1  = 0;

            IF (@V_IDUTCUSTOM1 = 'S') BEGIN

              SELECT
                 @V_VRPRECULISM1 = ISNULL(@V_MINPRECO, 0) / ISNULL(MAX(MOVINDICE.VRINDICE), 0)
              FROM
                 PARAMGERAL,
                 MOVINDICE
              WHERE
                 PARAMGERAL.CDINDICEMF = MOVINDICE.CDINDICE AND
                 MOVINDICE.DTREFIND    = @CUSTO_DTINIVALITM AND
                 MOVINDICE.CDTPINDICE  = '1' AND
                 MOVINDICE.IDPERVALOR  = '2';

            END;

            SET @V_VRPRECULISM2 = 0;

            IF (@V_IDUTCUSTOM2 = 'S') BEGIN

              SELECT
                 @V_VRPRECULISM2 = ISNULL(@V_MINPRECO, 0) / ISNULL(MAX(MOVINDICE.VRINDICE), 0)
              FROM
                 PARAMGERAL,
                 MOVINDICE
              WHERE
                 PARAMGERAL.CDINDICEMF2 = MOVINDICE.CDINDICE AND
                 MOVINDICE.DTREFIND     = @CUSTO_DTINIVALITM AND
                 MOVINDICE.CDTPINDICE   = '1' AND
                 MOVINDICE.IDPERVALOR   = '2';

            END;

            IF (@V_IDUTCUSLIPR = 'S') BEGIN
               SET @V_VRIMPITM = 0;
               DECLARE C_IMPOSTO CURSOR FOR
                  SELECT
                     IMPOSTO.CDIMPOSTO,
                     IMPOSTO.IDTRATIMPO,
                     IMPOSTO.IDIMPORECUP,
                     IMPOSTO.CDCCTBLATIV,
                     IMPOSTO.CDCCTBLPASS,
                     IMPCOLET.VRADIMPCOL,
                     IMPCOLET.VRALIQIMPCOL,
                     IMPCOLET.VRIMPCOLETA,
                     IMPCOLET.CDFISCITMCOL
                  FROM
                     IMPCOLET,
                     IMPOSTO
                  WHERE
                     IMPCOLET.CDFILIAL    = @P_CDFILIAL     AND
                     IMPCOLET.NRCOLEPRECO = @V_NRCOLEPRECO  AND
                     IMPCOLET.CDFORNECED  = @V_CDFORNECED   AND
                     IMPCOLET.CDREGIAO    = @V_CDREGIAO     AND
                     IMPCOLET.CDPRODUTO   = @V_CDPRODUTO    AND
                     IMPCOLET.CDMARCA     = @V_CDMARCA      AND
                     IMPCOLET.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
               OPEN C_IMPOSTO;
               FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                              @V_IMPOSTO_IDTRATIMPO,
                                              @V_IMPOSTO_IDIMPORECUP,
                                              @V_IMPOSTO_CDCCTBLATIV,
                                              @V_IMPOSTO_CDCCTBLPASS,
                                              @V_IMPOSTO_VRADIMPCOL,
                                              @V_IMPOSTO_VRALIQIMPCOL,
                                              @V_IMPOSTO_VRIMPCOLETA,
                                              @V_IMPOSTO_CDFISCITMCOL;
               WHILE (@@FETCH_STATUS = 0)  BEGIN
                  SELECT
                     @V_CDIMPOSTRIB  = MAX(CDIMPOSTO),
                     @V_IDVRIMPSTRIB = MAX(IDVRIMPSTRIB)
                  FROM
                     IMPOSTO
                  WHERE
                     CDIMPOSTRIB = @V_IMPOSTO_CDIMPOSTO;

                  IF (@V_IMPOSTO_IDIMPORECUP = 'S') BEGIN
                     IF (@V_IMPOSTO_CDFISCITMCOL = '1') BEGIN
                        IF (@V_IMPOSTO_CDCCTBLATIV IS NOT NULL) BEGIN
                           IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                              SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                           END;
                        END;
                     END
                     ELSE BEGIN
                        IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                           IF @V_CDIMPOSTRIB IS NULL BEGIN
                              SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                           END;
                        END
                        ELSE BEGIN
                           IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                              SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                           END;
                        END;
                     END;
                  END
                  ELSE BEGIN
                     IF @V_IMPOSTO_CDCCTBLPASS IS NOT NULL BEGIN
                        IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                           SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                        END;
                     END
                     ELSE BEGIN
                        IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                           IF @V_CDIMPOSTRIB IS NULL BEGIN
                              SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                           END;
                        END
                        ELSE BEGIN
                           IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                              SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                           END;
                        END;
                     END;
                  END;
                  FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                                 @V_IMPOSTO_IDTRATIMPO,
                                                 @V_IMPOSTO_IDIMPORECUP,
                                                 @V_IMPOSTO_CDCCTBLATIV,
                                                 @V_IMPOSTO_CDCCTBLPASS,
                                                 @V_IMPOSTO_VRADIMPCOL,
                                                 @V_IMPOSTO_VRALIQIMPCOL,
                                                 @V_IMPOSTO_VRIMPCOLETA,
                                                 @V_IMPOSTO_CDFISCITMCOL;
               END;
               CLOSE C_IMPOSTO;
               DEALLOCATE C_IMPOSTO;
               SELECT
                  @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
               FROM
                  PRODUTO
               WHERE
                  CDPRODUTO = @V_CDPRODUTO;
            END;
            INSERT INTO CUSTOSUP
               (CDFILIAL, CDPRODUTO, DTCUSTOSUP,
                VRCUSPRLISTP, CDFILCOTLIST, NRCOLEPRLIST,
                CDREGIAOLIST, CDFORNECLIST, CDPRODLIST,
                CDMARCALIST, VRCUSPRLISM1, VRCUSPRLISM2, VRHISPRLISTP, DTFINCUSTSUP, VRCUSLIPRIMP)
            VALUES
               (@CUSTO_CDFILIAL,
                @CUSTO_CDPRODESTO,
                @CUSTO_DTINIVALITM,
                @V_MINPRECO,
                @P_CDFILIAL,
                @V_NRCOLEPRECO,
                @V_CDREGIAO,
                @V_CDFORNECED,
                @V_CDPRODUTO,
                @V_CDMARCA,
                @V_VRPRECULISM1,
                @V_VRPRECULISM2,
                @V_MINPRECO,
                @CUSTO_DTFINVALITM,
                CASE @V_IDUTCUSLIPR
                   WHEN 'N'
                   THEN NULL
                   ELSE ((@V_MINPRECO*@V_VRFATOCONV) + @V_VRIMPITM)/@V_VRFATOCONV
                END
                );
          END
          ELSE BEGIN

             /* O campo VRHISPRLISTP, no update abaixo, tem a funo de guardar o custo da lista,
               para o caso dele ser substitudo pelo preo de algum acordo existente, no processo
               que  executado mais abaixo. */

             /*######################### REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ######################################*/
             DECLARE C_CUSLISTINFLAC CURSOR FOR /*RETORNA OS CUSTOS DE LISTA INFLACIONADA ANTERIORES AO QUE ESTA SENDO GRAVADO*/
                 SELECT
                    DTCUSTOSUP,
                    DTFINCUSTSUP
                 FROM
                    CUSTOSUP
                 WHERE
                    CDFILIAL = @CUSTO_CDFILIAL AND
                    CDPRODUTO = @CUSTO_CDPRODESTO AND
                    IDCUSLISTINF = 'S' AND
                    DTCUSTOSUP <= @CUSTO_DTFINVALITM;
             OPEN C_CUSLISTINFLAC;
             FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                                  @V_CUSLISTINFLAC_DTFINVALITM;
             WHILE (@@FETCH_STATUS = 0) BEGIN
                 IF @CUSTO_DTFINVALITM + 1 > @V_CUSLISTINFLAC_DTFINVALITM BEGIN /*SE O CUSTO DA LISTA INFLACIONADA VAI SER EXCLUIDO OU NAO*/
                    DELETE FROM CUSTOSUP
                    WHERE
                       CDFILIAL = @CUSTO_CDFILIAL AND
                       CDPRODUTO = @CUSTO_CDPRODESTO AND
                       DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                       IDCUSLISTINF = 'S';
                 END
                 ELSE BEGIN
                    UPDATE CUSTOSUP
                    SET DTCUSTOSUP = @CUSTO_DTFINVALITM + 1
                    WHERE
                       CDFILIAL = @CUSTO_CDFILIAL AND
                       CDPRODUTO = @CUSTO_CDPRODESTO AND
                       DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                       IDCUSLISTINF = 'S';
                 END;
                 FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                                      @V_CUSLISTINFLAC_DTFINVALITM;
             END;
             CLOSE C_CUSLISTINFLAC;
             DEALLOCATE C_CUSLISTINFLAC;
      /*######################### FIM DO REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ###############################*/
             IF @V_IDUTCUSLIPR = 'S' BEGIN
                SET @V_VRIMPITM = 0;
                DECLARE C_IMPOSTO CURSOR FOR
                   SELECT
                      IMPOSTO.CDIMPOSTO,
                      IMPOSTO.IDTRATIMPO,
                      IMPOSTO.IDIMPORECUP,
                      IMPOSTO.CDCCTBLATIV,
                      IMPOSTO.CDCCTBLPASS,
                      IMPCOLET.VRADIMPCOL,
                      IMPCOLET.VRALIQIMPCOL,
                      IMPCOLET.VRIMPCOLETA,
                      IMPCOLET.CDFISCITMCOL
                   FROM
                      IMPCOLET,
                      IMPOSTO
                   WHERE
                      IMPCOLET.CDFILIAL    = @P_CDFILIAL    AND
                      IMPCOLET.NRCOLEPRECO = @V_NRCOLEPRECO AND
                      IMPCOLET.CDFORNECED  = @V_CDFORNECED  AND
                      IMPCOLET.CDREGIAO    = @V_CDREGIAO    AND
                      IMPCOLET.CDPRODUTO   = @V_CDPRODUTO   AND
                      IMPCOLET.CDMARCA     = @V_CDMARCA     AND
                      IMPCOLET.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
                OPEN C_IMPOSTO;
                FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                               @V_IMPOSTO_IDTRATIMPO,
                                               @V_IMPOSTO_IDIMPORECUP,
                                               @V_IMPOSTO_CDCCTBLATIV,
                                               @V_IMPOSTO_CDCCTBLPASS,
                                               @V_IMPOSTO_VRADIMPCOL,
                                               @V_IMPOSTO_VRALIQIMPCOL,
                                               @V_IMPOSTO_VRIMPCOLETA,
                                               @V_IMPOSTO_CDFISCITMCOL;
                WHILE (@@FETCH_STATUS = 0)  BEGIN
                   SELECT
                      @V_CDIMPOSTRIB  = MAX(CDIMPOSTO),
                      @V_IDVRIMPSTRIB = MAX(IDVRIMPSTRIB)
                   FROM
                      IMPOSTO
                   WHERE
                      CDIMPOSTRIB = @V_IMPOSTO_CDIMPOSTO;

                   IF (@V_IMPOSTO_IDIMPORECUP = 'S') BEGIN
                      IF (@V_IMPOSTO_CDFISCITMCOL = '1') BEGIN
                         IF (@V_IMPOSTO_CDCCTBLATIV IS NOT NULL) BEGIN
                            IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                               SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                            END;
                         END;
                      END
                      ELSE BEGIN
                         IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                            IF @V_CDIMPOSTRIB IS NULL BEGIN
                               SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                            END;
                         END
                         ELSE BEGIN
                            IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                               SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                            END;
                         END;
                      END;
                   END
                   ELSE BEGIN
                      IF @V_IMPOSTO_CDCCTBLPASS IS NOT NULL BEGIN
                         IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                            SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                         END;
                      END
                      ELSE BEGIN
                         IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                            IF @V_CDIMPOSTRIB IS NULL BEGIN
                               SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                            END;
                         END
                         ELSE BEGIN
                            IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                               SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                            END;
                         END;
                      END;
                   END;
                   FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                                  @V_IMPOSTO_IDTRATIMPO,
                                                  @V_IMPOSTO_IDIMPORECUP,
                                                  @V_IMPOSTO_CDCCTBLATIV,
                                                  @V_IMPOSTO_CDCCTBLPASS,
                                                  @V_IMPOSTO_VRADIMPCOL,
                                                  @V_IMPOSTO_VRALIQIMPCOL,
                                                  @V_IMPOSTO_VRIMPCOLETA,
                                                  @V_IMPOSTO_CDFISCITMCOL;
                END;
                CLOSE C_IMPOSTO;
                DEALLOCATE C_IMPOSTO;
                SELECT
                   @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
                FROM
                   PRODUTO
                WHERE
                   CDPRODUTO = @V_CDPRODUTO;
             END;

             UPDATE CUSTOSUP
             SET VRCUSPRLISTP = @V_MINPRECO,
                 VRHISPRLISTP = @V_MINPRECO,
                 CDFILCOTLIST = @P_CDFILIAL,
                 NRCOLEPRLIST = @V_NRCOLEPRECO,
                 CDREGIAOLIST = @V_CDREGIAO,
                 CDFORNECLIST = @V_CDFORNECED,
                 CDPRODLIST   = @V_CDPRODUTO,
                 CDMARCALIST  = @V_CDMARCA,
                 DTFINCUSTSUP = @CUSTO_DTFINVALITM,

                 CDFILCOTACOR = NULL,
                 NRCOLEPRACOR = NULL,
                 CDMARCAACOR  = NULL,
                 CDREGIAOACOR = NULL,
                 CDFORNECACOR = NULL,
                 CDPRODACOR   = NULL,

                 VRCUSPRLISM1 = (SELECT ISNULL(MAX(MOVINDICE.VRINDICE), 0) * ISNULL(@V_MINPRECO, 0)
                                 FROM PARAMGERAL, MOVINDICE
                                 WHERE @V_IDUTCUSTOM1 = 'S' AND
                                       PARAMGERAL.CDINDICEMF = MOVINDICE.CDINDICE AND
                                       MOVINDICE.DTREFIND    = @CUSTO_DTINIVALITM AND
                                       MOVINDICE.CDTPINDICE  = '1' AND
                                       MOVINDICE.IDPERVALOR  = '2'),

                 VRCUSPRLISM2 = (SELECT ISNULL(MAX(MOVINDICE.VRINDICE), 0) * ISNULL(@V_MINPRECO, 0)
                                 FROM PARAMGERAL, MOVINDICE
                                 WHERE @V_IDUTCUSTOM2 = 'S' AND
                                       PARAMGERAL.CDINDICEMF2 = MOVINDICE.CDINDICE AND
                                       MOVINDICE.DTREFIND     = @CUSTO_DTINIVALITM AND
                                       MOVINDICE.CDTPINDICE   = '1' AND
                                       MOVINDICE.IDPERVALOR   = '2'),

                 VRCUSLIPRIMP = CASE @V_IDUTCUSLIPR
                               WHEN 'N' THEN NULL
                                        ELSE
                                        CASE @V_NRCOLEPRECO
                                        WHEN NULL THEN NULL
                                                  ELSE
                                                 ((@V_MINPRECO*@V_VRFATOCONV) + @V_VRIMPITM)/@V_VRFATOCONV
                                        END
                               END
             WHERE
               CDFILIAL   = @CUSTO_CDFILIAL   AND
               CDPRODUTO  = @CUSTO_CDPRODESTO AND
               DTCUSTOSUP = @CUSTO_DTINIVALITM;
          END;

          SET @V_AUX_CUSTO = @V_AUX_CUSTO - 1;
          IF @V_AUX_CUSTO < 0 BEGIN
             SET @V_AUX_CUSTO = 0;
          END;
          IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
             UPDATE TEMPULTATULISPR
             SET QTPRODLISPREC = @V_AUX_CUSTO,
                 CDPRODULLISPREC = @CUSTO_CDPRODESTO
             WHERE CDCENULTLISPREC = @P_CDFILIAL
               AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
          END;
          FETCH NEXT FROM C_CUSTO INTO @CUSTO_CDFILIAL, @CUSTO_DTINIVALITM, @CUSTO_DTFINVALITM, @CUSTO_CDPRODESTO, @CUSTO_VRMINPRECO, @CUSTO_VRPRECOPI, @CUSTO_MINPRECO
        END;
        CLOSE C_CUSTO;
        DEALLOCATE C_CUSTO;
        SET @V_AUX_FILIAL = @V_AUX_FILIAL - 1;
        IF @V_AUX_FILIAL < 0 BEGIN
           SET @V_AUX_FILIAL = 0;
        END;
        IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
           UPDATE TEMPULTATULISPR
           SET QTFILIALLISPREC = @V_AUX_FILIAL - 1,
               CDFILULTLISPREC = @V_FILIAL_CDFILIAL
           WHERE CDCENULTLISPREC = @P_CDFILIAL
             AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
        END;     
        FETCH NEXT FROM C_FILIAL INTO @V_FILIAL_CDFILIAL;
     END
     CLOSE C_FILIAL;
     DEALLOCATE C_FILIAL;
     SET @V_AUX_REGIAOGERAL = @V_AUX_REGIAOGERAL - 1;
     IF @V_AUX_REGIAOGERAL < 0 BEGIN
        SET @V_AUX_REGIAOGERAL = 0;
     END;
     IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
        UPDATE TEMPULTATULISPR
        SET QTREGIAOLISPREC = @V_AUX_REGIAOGERAL,
            CDREGULTLISPREC = @V_REGIAO_CDREGIAO
        WHERE CDCENULTLISPREC = @P_CDFILIAL
          AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
     END;     
     FETCH NEXT FROM C_REGIAO INTO @V_REGIAO_CDREGIAO;
  END;
  CLOSE C_REGIAO;
  DEALLOCATE C_REGIAO;


  IF (@P_CDLISTAREG IS NULL) BEGIN
     INSERT INTO CUSTOSUPACOR_TMP
      (CDFILIAL,
       CDFILCENTCMP,
       NRSEQPRELIST,
       DTINIVALITM,
       DTFINVALITM,
       CDPRODESTO,
       NRCOLEPRECO,
       CDREGIAO,
       CDFORNECED,
       CDPRODUTO,
       CDMARCA,
       VRPRECOUN,
       VRPRECECO,
       VRPRECOPI)
     SELECT /*+ INDEX (ITEMCOLE PK_ITEMCOLE) */
       FILIAL.CDFILIAL,
       @P_CDFILIAL,
       @P_NRSEQPRELIST,
       ITEMCOLE.DTINIVALITM,
       ITEMCOLE.DTFINVALITM,
       PRODUTO.CDPRODESTO,
       ITEMCOLE.NRCOLEPRECO,
       ITEMCOLE.CDREGIAO,
       ITEMCOLE.CDFORNECED,
       PRODUTO.CDPRODUTO,
       ITEMCOLE.CDMARCA,
       ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV AS VRPRECOUN,
       ITEMCOLE.VRPRECECOITM/PRODUTO.VRFATOCONV AS VRPRECECO,
       ITEMCOLE.VRPRECOITMPI/PRODUTO.VRFATOCONV AS VRPRECOPI
     FROM
       PRODUTO,
       FILIAL,
       COLETAPR,
       ITEMCOLE,
       PARTFILI
     WHERE
       PARTFILI.CDTIPPART   ='4'                                     AND
       ITEMCOLE.VRPRECUNIITM    > 0                                  AND
       ITEMCOLE.CDFILIAL    = COLETAPR.CDFILIAL                      AND
       ITEMCOLE.NRCOLEPRECO = COLETAPR.NRCOLEPRECO                   AND
       ITEMCOLE.DTFINVALITM >= CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103)) AND
       ITEMCOLE.CDFILIAL    = @P_CDFILIAL                            AND
       PARTFILI.CDFORNECED  = ITEMCOLE.CDFORNECED                    AND
       PARTFILI.CDFILCOTA   = @P_CDFILIAL                            AND
       PARTFILI.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO                   AND
       PARTFILI.CDPRODUTO   = ITEMCOLE.CDPRODUTO                     AND
       FILIAL.CDFILIAL      = ISNULL(@P_CDFILCUSTO, FILIAL.CDFILIAL) AND
       PARTFILI.CDFILIAL    = FILIAL.CDFILIAL                        AND
       ITEMCOLE.CDPRODUTO   = PRODUTO.CDPRODUTO                      AND
       ITEMCOLE.CDREGIAO    = FILIAL.CDREGIAO;
  END
  ELSE BEGIN
     INSERT INTO CUSTOSUPACOR_TMP
      (CDFILIAL,
       CDFILCENTCMP,
       NRSEQPRELIST,
       DTINIVALITM,
       DTFINVALITM,
       CDPRODESTO,
       NRCOLEPRECO,
       CDREGIAO,
       CDFORNECED,
       CDPRODUTO,
       CDMARCA,
       VRPRECOUN,
       VRPRECECO,
       VRPRECOPI)
     SELECT /*+ INDEX (ITEMCOLE PK_ITEMCOLE) */
       FILIAL.CDFILIAL,
       @P_CDFILIAL,
       @P_NRSEQPRELIST,
       ITEMCOLE.DTINIVALITM,
       ITEMCOLE.DTFINVALITM,
       PRODUTO.CDPRODESTO,
       ITEMCOLE.NRCOLEPRECO,
       ITEMCOLE.CDREGIAO,
       ITEMCOLE.CDFORNECED,
       PRODUTO.CDPRODUTO,
       ITEMCOLE.CDMARCA,
       ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV AS VRPRECOUN,
       ITEMCOLE.VRPRECECOITM/PRODUTO.VRFATOCONV AS VRPRECECO,
       ITEMCOLE.VRPRECOITMPI/PRODUTO.VRFATOCONV AS VRPRECOPI
     FROM
       PRODUTO,
       FILIAL,
       COLETAPR,
       ITEMCOLE,
       PARTFILI,
       LISTA LISTAREG
     WHERE
       PARTFILI.CDTIPPART   ='4'                                     AND
       ITEMCOLE.VRPRECUNIITM    > 0                                  AND
       ITEMCOLE.CDFILIAL    = COLETAPR.CDFILIAL                      AND
       ITEMCOLE.NRCOLEPRECO = COLETAPR.NRCOLEPRECO                   AND
       ITEMCOLE.DTFINVALITM >= CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103)) AND
       ITEMCOLE.CDFILIAL    = @P_CDFILIAL                            AND
       PARTFILI.CDFORNECED  = ITEMCOLE.CDFORNECED                    AND
       PARTFILI.CDFILCOTA   = @P_CDFILIAL                            AND
       PARTFILI.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO                   AND
       PARTFILI.CDPRODUTO   = ITEMCOLE.CDPRODUTO                     AND
       FILIAL.CDFILIAL      = ISNULL(@P_CDFILCUSTO, FILIAL.CDFILIAL) AND
       PARTFILI.CDFILIAL    = FILIAL.CDFILIAL                        AND
       ITEMCOLE.CDPRODUTO   = PRODUTO.CDPRODUTO                      AND
       LISTAREG.NRLISTA     = @P_CDLISTAREG                          AND
       LISTAREG.CDLISTA     = ITEMCOLE.CDREGIAO;
  END;

  DECLARE C_ACORDO CURSOR FOR
    (SELECT  /*+ INDEX (PRODPREFMED PK_PRODPREFMED) */
       CUSTO.CDFILIAL,
       CUSTO.DTINIVALITM,
       CUSTO.DTFINVALITM,
       CUSTO.CDPRODESTO,
       MIN(
           RIGHT(REPLICATE('0',23)+ CASE @V_IDTIPOAPUR
                                        WHEN NULL THEN CUSTO.VRPRECOUN
                                        WHEN '1'  THEN CUSTO.VRPRECECO
                                        ELSE CUSTO.VRPRECOPI
                                    END * @V_MULTIPLO, 23) +'#'+
           RIGHT(REPLICATE('0', 23)+CUSTO.VRPRECOUN * @V_MULTIPLO, 23) +'%'+
           CUSTO.NRCOLEPRECO +
           CUSTO.CDREGIAO    +
           CUSTO.CDPRODUTO   +
           CUSTO.CDMARCA     +
           CUSTO.CDFORNECED) AS MINPRECO
     FROM
       CUSTOSUPACOR_TMP CUSTO LEFT JOIN PRODPREFMED
                                     ON CUSTO.CDFILIAL   = PRODPREFMED.CDFILIAL AND
                                        CUSTO.CDPRODESTO = PRODPREFMED.CDPRODUTO,
       COLETAPR,
       FORNECEDOR
     WHERE
       @P_CDFILIAL        = COLETAPR.CDFILIAL      AND
       CUSTO.NRCOLEPRECO  = COLETAPR.NRCOLEPRECO   AND
       CUSTO.CDPRODUTO    = ISNULL(PRODPREFMED.CDPRODPREF, CUSTO.CDPRODUTO) AND
       CUSTO.CDFORNECED   = FORNECEDOR.CDFORNECED  AND
       FORNECEDOR.IDSITUCADA   = 'A'               AND
       ISNULL(FORNECEDOR.IDTIPOFORN, 'N') <> 'A'   AND

       NOT EXISTS
        (SELECT
           CDFILIAL
         FROM
           REFORFIL
         WHERE
           CDFILIAL   = CUSTO.CDFILIAL    AND
           CDFORNECED = CUSTO.CDFORNECED AND
           CDPERIENTR IS NULL) AND


       NOT EXISTS
        (SELECT
           REFOREG.CDREGIAO
         FROM
           REFOREG
         WHERE
           REFOREG.CDREGIAO   = CUSTO.CDREGIAO AND
           REFOREG.CDFORNECED = CUSTO.CDFORNECED) AND


       NOT EXISTS
        (SELECT
            QFOPRODFORN.CDPRODUTO
         FROM
            QFOPRODFORN,
            PARASUPRI
         WHERE
           PARASUPRI.CDFILIAL = @P_CDFILIAL  AND
           (QFOPRODFORN.CDPRODUTO = CUSTO.CDPRODUTO) AND
           (QFOPRODFORN.CDFORNECED= CUSTO.CDFORNECED) AND
           ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR
            ((QFOPRODFORN.IDFORNAPROV = 'S') AND
               ((PARASUPRI.IDULTIDTVALQ = 'M' AND
                 COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN) - DAY(DATEADD(MM, 1, QFOPRODFORN.DTVALPRFORN))) OR
                (PARASUPRI.IDULTIDTVALQ = 'D' AND
                 COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN)) ) )) AND

       NOT EXISTS
         (SELECT
            QFOMARPROD.CDMARCA
          FROM
            QFOMARPROD, PARASUPRI
          WHERE
           (PARASUPRI.CDFILIAL = @P_CDFILIAL)            AND
           (QFOMARPROD.CDPRODUTO = CUSTO.CDPRODUTO) AND
           (QFOMARPROD.CDMARCA   = CUSTO.CDMARCA)     AND
           ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR
            ((QFOMARPROD.IDAPRMARPRO = 'S') AND
               ((PARASUPRI.IDULTIDTVALQ = 'M' AND
                 COLETAPR.DTCOLEPREC > DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA) - DAY(DATEADD(MM, 1, QFOMARPROD.DTVALAPROVA))) OR
                (PARASUPRI.IDULTIDTVALQ = 'D' AND
                 COLETAPR.DTCOLEPREC > QFOMARPROD.DTVALAPROVA)) ) )) AND

       EXISTS
         (SELECT
            QFOMARPROD.CDMARCA
          FROM
            QFOMARPROD
          WHERE
            QFOMARPROD.CDPRODUTO = CUSTO.CDPRODUTO AND
            QFOMARPROD.CDMARCA   = CUSTO.CDMARCA

          UNION

          SELECT
            PARASUPRI.CDMARCA
          FROM
            PARASUPRI
          WHERE
            PARASUPRI.CDFILIAL    = @P_CDFILIAL AND
            (PARASUPRI.IDMARSEMQUA = 'N' OR PARASUPRI.IDMARSEMQUA IS NULL)) AND

       NOT EXISTS
         (SELECT QUALIFICACAO.CDQUALIF
          FROM QUALIFICACAO, PARASUPRI
          WHERE (FORNECEDOR.CDQUALIF       = QUALIFICACAO.CDQUALIF) AND
                (PARASUPRI.CDFILIAL        = @P_CDFILIAL)           AND
                (FORNECEDOR.IDFORNAUTO     = 'N')                   AND
                ((QUALIFICACAO.IDPERCOTCMP = '2')                   OR
                 ((PARASUPRI.IDULTIDTVALQ  = 'D'                    AND
                   COLETAPR.DTCOLEPREC > FORNECEDOR.DTVALQUALIF)       OR
                 (PARASUPRI.IDULTIDTVALQ = 'M'                      AND
                  COLETAPR.DTCOLEPREC > DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF) - DAY(DATEADD(MM, 1, FORNECEDOR.DTVALQUALIF)))))) AND
       (
                 (EXISTS (SELECT
                        1
                    FROM
                        PARTFILI
                    WHERE
                        (CDFILIAL  = CUSTO.CDFILIAL)    AND
                        (CDPRODUTO = CUSTO.CDPRODUTO) AND
                        (CDTIPPART IN ('1', '2'))        AND
                        (CDMARCA   = CUSTO.CDMARCA)   AND
                        (NOT EXISTS (SELECT
                                        1
                                     FROM
                                        PARTFILI
                                     WHERE
                                        (CDFILIAL   = CUSTO.CDFILIAL ) AND
                                        (CDPRODUTO  = CUSTO.CDPRODUTO) AND
                                        (CDTIPPART  IN ('4', '5'))            AND
                                        (CDFORNECED <> CUSTO.CDFORNECED))

                         )
                      )



                   )

                  OR (EXISTS (SELECT
                                 1
                              FROM
                                 PARTFILI
                              WHERE
                                 (CDFILIAL   = CUSTO.CDFILIAL )    AND
                                 (CDPRODUTO  = CUSTO.CDPRODUTO)  AND
                                 (CDTIPPART  IN ('4', '5'))         AND
                                 (CDFORNECED = CUSTO.CDFORNECED) AND
                                 (NOT EXISTS (SELECT
                                                 1
                                              FROM
                                                 PARTFILI
                                              WHERE
                                                (CDFILIAL  = CUSTO.CDFILIAL)    AND
                                                (CDPRODUTO = CUSTO.CDPRODUTO) AND
                                                (CDTIPPART IN ('1', '2'))        AND
                                                (CDMARCA   <> CUSTO.CDMARCA))
                                   )
                                )

                      ) OR

        (EXISTS (SELECT
                    CDFORNECED
                 FROM
                    PARTFILI
                 WHERE
                    (CDFILIAL  = CUSTO.CDFILIAL)           AND
                    (CDPRODUTO = CUSTO.CDPRODUTO) AND
                    (CDTIPPART  = '7')                   AND
                    (CDMARCA    = CUSTO.CDMARCA)      AND
                    (CDFORNECED = CUSTO.CDFORNECED))     ) OR
        (EXISTS (SELECT
                    CDFORNECED
                 FROM
                    PARTFILI
                 WHERE
                    (CDFILIAL  = CUSTO.CDFILIAL)           AND
                    (CDPRODUTO = CUSTO.CDPRODUTO) AND
                    (CDTIPPART  = '8')                   AND
                    (CDMARCA    = CUSTO.CDMARCA)      AND
                    (CDFORNECED = CUSTO.CDFORNECED))     ) OR
             (EXISTS (SELECT
                         CDPRODUTO
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTO.CDFILIAL)  AND
                         (CDPRODUTO = CUSTO.CDPRODUTO) AND
                         (CDTIPPART  IN ('3','6','9')))       AND
                         NOT EXISTS (SELECT
                                        1
                                     FROM
                                        PARTFILI
                                     WHERE
                                        (CDFILIAL  = CUSTO.CDFILIAL)    AND
                                        (CDPRODUTO = CUSTO.CDPRODUTO) AND
                                        (CDTIPPART  IN ('1','2','4','5','7','8')))



                          ) OR
             (NOT EXISTS (SELECT
                         CDPRODUTO
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTO.CDFILIAL)           AND
                         (CDPRODUTO = CUSTO.CDPRODUTO))    )

       ) AND
         (NOT EXISTS (SELECT
                         CDMARCA
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTO.CDFILIAL)           AND
                         (CDPRODUTO = CUSTO.CDPRODUTO) AND
                         (CDTIPPART = '3')                   AND
                         (CDMARCA   = CUSTO.CDMARCA))         )   AND
         (NOT EXISTS (SELECT
                         CDFORNECED
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTO.CDFILIAL)           AND
                         (CDPRODUTO = CUSTO.CDPRODUTO) AND
                         (CDTIPPART  = '6')                   AND
                         (CDFORNECED = CUSTO.CDFORNECED))     )   AND
         (NOT EXISTS (SELECT
                         CDFORNECED
                      FROM
                         PARTFILI
                      WHERE
                         (CDFILIAL  = CUSTO.CDFILIAL)           AND
                         (CDPRODUTO = CUSTO.CDPRODUTO) AND
                         (CDTIPPART  = '9')                   AND
                         (CDMARCA    = CUSTO.CDMARCA)      AND
                         (CDFORNECED = CUSTO.CDFORNECED))     )
     GROUP BY
       CUSTO.CDFILIAL,
       CUSTO.DTINIVALITM,
       CUSTO.DTFINVALITM,
       CUSTO.CDPRODESTO)
  OPEN C_ACORDO;
  FETCH NEXT FROM C_ACORDO INTO @ACORDO_CDFILIAL, @ACORDO_DTINIVALITM, @ACORDO_DTFINVALITM, @ACORDO_CDPRODESTO, @ACORDO_MINPRECO;
  WHILE (@@FETCH_STATUS = 0)
  BEGIN
    SET @V_COUNT = 0;
    SELECT
       @V_COUNT = COUNT(*)
    FROM
       CUSTOSUP
    WHERE
       CDFILIAL   = @ACORDO_CDFILIAL   AND
       CDPRODUTO  = @ACORDO_CDPRODESTO AND
       DTCUSTOSUP = @ACORDO_DTINIVALITM;

    SET @V_POSMINPREPI = CHARINDEX('#', @CUSTO_MINPRECO);
    SET @V_POSMINPRECO = CHARINDEX('%', @CUSTO_MINPRECO);

    SET @V_MINPRECO    = CONVERT(DOUBLE PRECISION, (SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPREPI + 1, @V_POSMINPRECO - @V_POSMINPREPI - 1)))/@V_MULTIPLO;
    SET @V_NRCOLEPRECO = SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPRECO+1, 10);
    SET @V_CDREGIAO    = SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPRECO+11, 3);
    SET @V_CDPRODUTO   = SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPRECO+14, 10);
    SET @V_CDMARCA     = SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPRECO+24, 5);
    SET @V_CDFORNECED  = SUBSTRING(@CUSTO_MINPRECO, @V_POSMINPRECO+29, 21);

    SET @V_VRPERAJFORNFIL = NULL;
    SET @V_IDCONSPERPIAJPR = 'N';
    SELECT
       @V_VRPERAJFORNFIL = MAX(VRPERAJFORNFIL),
       @V_IDCONSPERPIAJPR = MAX(IDCONSPERPIAJPR)
    FROM AJUSTEPRFORNFIL
    WHERE AJUSTEPRFORNFIL.CDFORNECED = @V_CDFORNECED
      AND AJUSTEPRFORNFIL.CDFILIAL   = @ACORDO_CDFILIAL
      AND AJUSTEPRFORNFIL.DTINIAJPRFORFIL = (SELECT MAX(A.DTINIAJPRFORFIL)
                                             FROM AJUSTEPRFORNFIL A
                                             WHERE A.CDFORNECED = @V_CDFORNECED
                                               AND A.CDFILIAL = @ACORDO_CDFILIAL

                                               AND CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103)
                                                   BETWEEN A.DTINIAJPRFORFIL AND ISNULL(A.DTFINAJPRFORFIL, CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103)));

    SET @V_PIDSV = NULL;
    IF @V_IDCONSPERPIAJPR = 'S' BEGIN
       SELECT
          @V_PIDSV = MAX(VRPERPIPRFO)
       FROM PRODFORNCOMP
       WHERE PRODFORNCOMP.CDFORNECED  = @V_CDFORNECED
         AND PRODFORNCOMP.CDPRODCOMPL = @V_CDPRODUTO;

       IF @V_PIDSV IS NULL BEGIN
          SELECT @V_PIDSV = MAX(VRPERPIFORN)
          FROM FORNECEDOR
          WHERE FORNECEDOR.CDFORNECED = @V_CDFORNECED;
       END;
    END;

    IF @V_PIDSV IS NULL BEGIN
       SET @V_PIDSV = 0;
    END;
    IF @V_VRPERAJFORNFIL IS NULL BEGIN
       SET @V_VRPERAJFORNFIL = 0;
    END;

    SET @V_MINPRECO = @V_MINPRECO - (@V_MINPRECO * @V_PIDSV/100) + (@V_MINPRECO * @V_VRPERAJFORNFIL/100);

    IF @V_COUNT = 0 BEGIN

       SET @V_VRPRECULISM1  = 0;

       IF (@V_IDUTCUSTOM1 = 'S') BEGIN

          SELECT
             @V_VRPRECULISM1 = ISNULL(@V_MINPRECO, 0) / ISNULL(MAX(MOVINDICE.VRINDICE), 0)
          FROM
             PARAMGERAL,
             MOVINDICE
          WHERE
             PARAMGERAL.CDINDICEMF = MOVINDICE.CDINDICE AND
             MOVINDICE.DTREFIND    = @ACORDO_DTINIVALITM AND
             MOVINDICE.CDTPINDICE  = '1' AND
             MOVINDICE.IDPERVALOR  = '2';
       END;

       SET @V_VRPRECULISM2 = 0;

       IF (@V_IDUTCUSTOM2 = 'S') BEGIN

          SELECT
             @V_VRPRECULISM2 = ISNULL(@V_MINPRECO, 0) / ISNULL(MAX(MOVINDICE.VRINDICE), 0)
          FROM
             PARAMGERAL,
             MOVINDICE
          WHERE
             PARAMGERAL.CDINDICEMF2 = MOVINDICE.CDINDICE AND
             MOVINDICE.DTREFIND     = @ACORDO_DTINIVALITM AND
             MOVINDICE.CDTPINDICE   = '1' AND
             MOVINDICE.IDPERVALOR   = '2';

       END;

       IF (@V_IDUTCUSLIPR = 'S') BEGIN
          SET @V_VRIMPITM = 0;
          DECLARE C_IMPOSTO CURSOR FOR
             SELECT
                IMPOSTO.CDIMPOSTO,
                IMPOSTO.IDTRATIMPO,
                IMPOSTO.IDIMPORECUP,
                IMPOSTO.CDCCTBLATIV,
                IMPOSTO.CDCCTBLPASS,
                IMPCOLET.VRADIMPCOL,
                IMPCOLET.VRALIQIMPCOL,
                IMPCOLET.VRIMPCOLETA,
                IMPCOLET.CDFISCITMCOL
             FROM
                IMPCOLET,
                IMPOSTO
             WHERE
                IMPCOLET.CDFILIAL    = @P_CDFILIAL     AND
                IMPCOLET.NRCOLEPRECO = @V_NRCOLEPRECO  AND
                IMPCOLET.CDFORNECED  = @V_CDFORNECED   AND
                IMPCOLET.CDREGIAO    = @V_CDREGIAO     AND
                IMPCOLET.CDPRODUTO   = @V_CDPRODUTO    AND
                IMPCOLET.CDMARCA     = @V_CDMARCA      AND
                IMPCOLET.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
          OPEN C_IMPOSTO;
          FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                         @V_IMPOSTO_IDTRATIMPO,
                                         @V_IMPOSTO_IDIMPORECUP,
                                         @V_IMPOSTO_CDCCTBLATIV,
                                         @V_IMPOSTO_CDCCTBLPASS,
                                         @V_IMPOSTO_VRADIMPCOL,
                                         @V_IMPOSTO_VRALIQIMPCOL,
                                         @V_IMPOSTO_VRIMPCOLETA,
                                         @V_IMPOSTO_CDFISCITMCOL;
          WHILE (@@FETCH_STATUS = 0)  BEGIN
             SELECT
                @V_CDIMPOSTRIB  = MAX(CDIMPOSTO),
                @V_IDVRIMPSTRIB = MAX(IDVRIMPSTRIB)
             FROM
                IMPOSTO
             WHERE
                CDIMPOSTRIB = @V_IMPOSTO_CDIMPOSTO;

             IF (@V_IMPOSTO_IDIMPORECUP = 'S') BEGIN
                IF (@V_IMPOSTO_CDFISCITMCOL = '1') BEGIN
                   IF (@V_IMPOSTO_CDCCTBLATIV IS NOT NULL) BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END
                ELSE BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                      IF @V_CDIMPOSTRIB IS NULL BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END
                   ELSE BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END;
             END
             ELSE BEGIN
                IF @V_IMPOSTO_CDCCTBLPASS IS NOT NULL BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                      SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                   END;
                END
                ELSE BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                      IF @V_CDIMPOSTRIB IS NULL BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END
                   ELSE BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END;
             END;
             FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                            @V_IMPOSTO_IDTRATIMPO,
                                            @V_IMPOSTO_IDIMPORECUP,
                                            @V_IMPOSTO_CDCCTBLATIV,
                                            @V_IMPOSTO_CDCCTBLPASS,
                                            @V_IMPOSTO_VRADIMPCOL,
                                            @V_IMPOSTO_VRALIQIMPCOL,
                                            @V_IMPOSTO_VRIMPCOLETA,
                                            @V_IMPOSTO_CDFISCITMCOL;
          END;
          CLOSE C_IMPOSTO;
          DEALLOCATE C_IMPOSTO;
          SELECT
             @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
          FROM
             PRODUTO
          WHERE
             CDPRODUTO = @V_CDPRODUTO;
       END;

/*######################### REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ######################################*/
       DECLARE C_CUSLISTINFLAC CURSOR FOR /*RETORNA OS CUSTOS DE LISTA INFLACIONADA ANTERIORES AO QUE ESTA SENDO GRAVADO*/
           SELECT
              DTCUSTOSUP,
              DTFINCUSTSUP
           FROM
              CUSTOSUP
           WHERE
              CDFILIAL = @ACORDO_CDFILIAL AND
              CDPRODUTO = @ACORDO_CDPRODESTO AND
              IDCUSLISTINF = 'S' AND
              DTCUSTOSUP <= @ACORDO_DTFINVALITM;
       OPEN C_CUSLISTINFLAC;
       FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                            @V_CUSLISTINFLAC_DTFINVALITM;
       WHILE (@@FETCH_STATUS = 0) BEGIN
           IF @ACORDO_DTFINVALITM + 1 > @V_CUSLISTINFLAC_DTFINVALITM BEGIN /*SE O CUSTO DA LISTA INFLACIONADA VAI SER EXCLUIDO OU NAO*/
              DELETE FROM CUSTOSUP
              WHERE
                 CDFILIAL = @ACORDO_CDFILIAL AND
                 CDPRODUTO = @ACORDO_CDPRODESTO AND
                 DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                 IDCUSLISTINF = 'S';
           END
           ELSE BEGIN
              UPDATE CUSTOSUP
              SET DTCUSTOSUP = @ACORDO_DTFINVALITM + 1
              WHERE
                 CDFILIAL = @ACORDO_CDFILIAL AND
                 CDPRODUTO = @ACORDO_CDPRODESTO AND
                 DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                 IDCUSLISTINF = 'S';
           END;
           FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                                @V_CUSLISTINFLAC_DTFINVALITM;
       END;
       CLOSE C_CUSLISTINFLAC;
       DEALLOCATE C_CUSLISTINFLAC;
/*######################### FIM DO REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ###############################*/

       /* Insere um novo registro de custo de lista de preos - custo de acordo */
       INSERT INTO CUSTOSUP
          (CDFILIAL, CDPRODUTO, DTCUSTOSUP,
           VRCUSPRLISTP, CDFILCOTACOR, NRCOLEPRACOR,
           CDREGIAOACOR, CDFORNECACOR, CDPRODACOR,
           CDMARCAACOR, VRCUSPRLISM1, VRCUSPRLISM2, DTFINCUSTSUP, VRCUSLIPRIMP)
       VALUES
          (@ACORDO_CDFILIAL,
           @ACORDO_CDPRODESTO,
           @ACORDO_DTINIVALITM,
           @V_MINPRECO,
           @P_CDFILIAL,
           @V_NRCOLEPRECO,
           @V_CDREGIAO,
           @V_CDFORNECED,
           @V_CDPRODUTO,
           @V_CDMARCA,
           @V_VRPRECULISM1,
           @V_VRPRECULISM2,
           @ACORDO_DTFINVALITM,
           CASE @V_IDUTCUSLIPR
           WHEN 'N' THEN NULL
                    ELSE
                    CASE @V_NRCOLEPRECO
                    WHEN NULL THEN NULL
                              ELSE
                             ((@V_MINPRECO*@V_VRFATOCONV) + @V_VRIMPITM)/@V_VRFATOCONV
                    END
           END
           );
    END
    ELSE BEGIN

       /* Deletando custo imediatamente posterior ao do acordo */
       DELETE FROM CUSTOSUP
       WHERE
          CDFILIAL   = @ACORDO_CDFILIAL   AND
          CDPRODUTO  = @ACORDO_CDPRODESTO AND
          DTCUSTOSUP = @ACORDO_DTFINVALITM + 1;

       DECLARE C_CUSTOPOST CURSOR FOR
           SELECT
              CUSTOSUP.CDFILIAL,
              CUSTOSUP.CDPRODUTO,
              @ACORDO_DTFINVALITM + 1 AS DTCUSTOSUP,
              CUSTOSUP.VRCUSPRLISTP,
              CUSTOSUP.CDFILCOTLIST,
              CUSTOSUP.NRCOLEPRLIST,
              CUSTOSUP.CDREGIAOLIST,
              CUSTOSUP.CDFORNECLIST,
              CUSTOSUP.CDPRODLIST,
              CUSTOSUP.CDMARCALIST,
              CUSTOSUP.VRCUSPRLISM1,
              CUSTOSUP.VRCUSPRLISM2,
              CUSTOSUP.VRHISPRLISTP,
              PREVLISTA.DTFINVALLIST
           FROM
              CUSTOSUP,
              PREVLISTA
           WHERE
              (CUSTOSUP.CDFILIAL      = @ACORDO_CDFILIAL)    AND
              (CUSTOSUP.CDPRODUTO     = @ACORDO_CDPRODESTO)  AND
              (CUSTOSUP.DTCUSTOSUP    = @ACORDO_DTINIVALITM) AND
              (PREVLISTA.CDFILIAL     = @P_CDFILIAL)           AND
              (PREVLISTA.NRSEQPRELIST = @P_NRSEQPRELIST)       AND
              (PREVLISTA.DTINIVALLIST = @ACORDO_DTINIVALITM) AND
              (PREVLISTA.DTFINVALLIST > @ACORDO_DTFINVALITM);
       OPEN C_CUSTOPOST;
       FETCH NEXT FROM C_CUSTOPOST INTO @V_CUSTOPOST_CDFILIAL,
                                        @V_CUSTOPOST_CDPRODUTO,
                                        @V_CUSTOPOST_DTCUSTOSUP,
                                        @V_CUSTOPOST_VRCUSPRLISTP,
                                        @V_CUSTOPOST_CDFILCOTLIST,
                                        @V_CUSTOPOST_NRCOLEPRLIST,
                                        @V_CUSTOPOST_CDREGIAOLIST,
                                        @V_CUSTOPOST_CDFORNECLIST,
                                        @V_CUSTOPOST_CDPRODLIST,
                                        @V_CUSTOPOST_CDMARCALIST,
                                        @V_CUSTOPOST_VRCUSPRLISM1,
                                        @V_CUSTOPOST_VRCUSPRLISM2,
                                        @V_CUSTOPOST_VRHISPRLISTP,
                                        @V_CUSTOPOST_DTFINVALLIST;
       WHILE (@@FETCH_STATUS = 0) BEGIN
/*######################### REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ######################################*/
          DECLARE C_CUSLISTINFLAC CURSOR FOR /*RETORNA OS CUSTOS DE LISTA INFLACIONADA ANTERIORES AO QUE ESTA SENDO GRAVADO*/
              SELECT
                 DTCUSTOSUP,
                 DTFINCUSTSUP
              FROM
                 CUSTOSUP
              WHERE
                 CDFILIAL = @V_CUSTOPOST_CDFILIAL AND
                 CDPRODUTO = @V_CUSTOPOST_CDPRODUTO AND
                 IDCUSLISTINF = 'S' AND
                 DTCUSTOSUP <= @V_CUSTOPOST_DTFINVALLIST;
          OPEN C_CUSLISTINFLAC;
          FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                               @V_CUSLISTINFLAC_DTFINVALITM;
          WHILE (@@FETCH_STATUS = 0) BEGIN
              IF @V_CUSTOPOST_DTFINVALLIST + 1 > @V_CUSLISTINFLAC_DTFINVALITM BEGIN /*SE O CUSTO DA LISTA INFLACIONADA VAI SER EXCLUIDO OU NAO*/
                 DELETE FROM CUSTOSUP
                 WHERE
                    CDFILIAL = @V_CUSTOPOST_CDFILIAL AND
                    CDPRODUTO = @V_CUSTOPOST_CDPRODUTO AND
                    DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                    IDCUSLISTINF = 'S';
              END
              ELSE BEGIN
                 UPDATE CUSTOSUP
                 SET DTCUSTOSUP = @V_CUSTOPOST_DTFINVALLIST + 1
                 WHERE
                    CDFILIAL = @V_CUSTOPOST_CDFILIAL AND
                    CDPRODUTO = @V_CUSTOPOST_CDPRODUTO AND
                    DTCUSTOSUP = @V_CUSLISTINFLAC_DTCUSTOSUP AND
                    IDCUSLISTINF = 'S';
              END;
              FETCH NEXT FROM C_CUSLISTINFLAC INTO @V_CUSLISTINFLAC_DTCUSTOSUP,
                                                   @V_CUSLISTINFLAC_DTFINVALITM;
          END;
          CLOSE C_CUSLISTINFLAC;
          DEALLOCATE C_CUSLISTINFLAC;

/*######################### FIM DO REMANEJAMENTO DO CUSTO DA LISTA INFLACIONADA ###############################*/

       /* Caso j exista um custo de lista de preos com data de
          inicio igual ao custo de acordo e com a data final maior,
          ou seja, se o custo de acordo estiver dentro do perodo de
          uma lista, ento o custo de lista passar a ter a data
          de incio um dia depois da data final do custo de acordo */

          INSERT INTO CUSTOSUP
            (CDFILIAL,
             CDPRODUTO,
             DTCUSTOSUP,
             VRCUSPRLISTP,
             CDFILCOTLIST,
             NRCOLEPRLIST,
             CDREGIAOLIST,
             CDFORNECLIST,
             CDPRODLIST,
             CDMARCALIST,
             VRCUSPRLISM1,
             VRCUSPRLISM2,
             VRHISPRLISTP,
             DTFINCUSTSUP)
          VALUES
            (@V_CUSTOPOST_CDFILIAL,
             @V_CUSTOPOST_CDPRODUTO,
             @V_CUSTOPOST_DTCUSTOSUP,
             @V_CUSTOPOST_VRCUSPRLISTP,
             @V_CUSTOPOST_CDFILCOTLIST,
             @V_CUSTOPOST_NRCOLEPRLIST,
             @V_CUSTOPOST_CDREGIAOLIST,
             @V_CUSTOPOST_CDFORNECLIST,
             @V_CUSTOPOST_CDPRODLIST,
             @V_CUSTOPOST_CDMARCALIST,
             @V_CUSTOPOST_VRCUSPRLISM1,
             @V_CUSTOPOST_VRCUSPRLISM2,
             @V_CUSTOPOST_VRHISPRLISTP,
             @V_CUSTOPOST_DTFINVALLIST);
          FETCH NEXT FROM C_CUSTOPOST INTO @V_CUSTOPOST_CDFILIAL,
                                           @V_CUSTOPOST_CDPRODUTO,
                                           @V_CUSTOPOST_DTCUSTOSUP,
                                           @V_CUSTOPOST_VRCUSPRLISTP,
                                           @V_CUSTOPOST_CDFILCOTLIST,
                                           @V_CUSTOPOST_NRCOLEPRLIST,
                                           @V_CUSTOPOST_CDREGIAOLIST,
                                           @V_CUSTOPOST_CDFORNECLIST,
                                           @V_CUSTOPOST_CDPRODLIST,
                                           @V_CUSTOPOST_CDMARCALIST,
                                           @V_CUSTOPOST_VRCUSPRLISM1,
                                           @V_CUSTOPOST_VRCUSPRLISM2,
                                           @V_CUSTOPOST_VRHISPRLISTP,
                                           @V_CUSTOPOST_DTFINVALLIST;
       END;
       CLOSE C_CUSTOPOST;
       DEALLOCATE C_CUSTOPOST;

       /* Atualizando custo de acordo */


       IF (@V_IDUTCUSLIPR = 'S') BEGIN
          SET @V_VRIMPITM = 0;
          DECLARE C_IMPOSTO CURSOR FOR
             SELECT
                IMPOSTO.CDIMPOSTO,
                IMPOSTO.IDTRATIMPO,
                IMPOSTO.IDIMPORECUP,
                IMPOSTO.CDCCTBLATIV,
                IMPOSTO.CDCCTBLPASS,
                IMPCOLET.VRADIMPCOL,
                IMPCOLET.VRALIQIMPCOL,
                IMPCOLET.VRIMPCOLETA,
                IMPCOLET.CDFISCITMCOL
             FROM
                IMPCOLET,
                IMPOSTO
             WHERE
                IMPCOLET.CDFILIAL    = @P_CDFILIAL     AND
                IMPCOLET.NRCOLEPRECO = @V_NRCOLEPRECO  AND
                IMPCOLET.CDFORNECED  = @V_CDFORNECED   AND
                IMPCOLET.CDREGIAO    = @V_CDREGIAO     AND
                IMPCOLET.CDPRODUTO   = @V_CDPRODUTO    AND
                IMPCOLET.CDMARCA     = @V_CDMARCA      AND
                IMPCOLET.CDIMPOSTO   = IMPOSTO.CDIMPOSTO
          OPEN C_IMPOSTO;
          FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                         @V_IMPOSTO_IDTRATIMPO,
                                         @V_IMPOSTO_IDIMPORECUP,
                                         @V_IMPOSTO_CDCCTBLATIV,
                                         @V_IMPOSTO_CDCCTBLPASS,
                                         @V_IMPOSTO_VRADIMPCOL,
                                         @V_IMPOSTO_VRALIQIMPCOL,
                                         @V_IMPOSTO_VRIMPCOLETA,
                                         @V_IMPOSTO_CDFISCITMCOL;
          WHILE (@@FETCH_STATUS = 0)  BEGIN
             SELECT
                @V_CDIMPOSTRIB  = MAX(CDIMPOSTO),
                @V_IDVRIMPSTRIB = MAX(IDVRIMPSTRIB)
             FROM
                IMPOSTO
             WHERE
                CDIMPOSTRIB = @V_IMPOSTO_CDIMPOSTO;

             IF (@V_IMPOSTO_IDIMPORECUP = 'S') BEGIN
                IF (@V_IMPOSTO_CDFISCITMCOL = '1') BEGIN
                   IF (@V_IMPOSTO_CDCCTBLATIV IS NOT NULL) BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END
                ELSE BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                      IF @V_CDIMPOSTRIB IS NULL BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END
                   ELSE BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END;
             END
             ELSE BEGIN
                IF @V_IMPOSTO_CDCCTBLPASS IS NOT NULL BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'V' BEGIN
                      SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                   END;
                END
                ELSE BEGIN
                   IF @V_IMPOSTO_IDTRATIMPO = 'A' BEGIN
                      IF @V_CDIMPOSTRIB IS NULL BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM + @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END
                   ELSE BEGIN
                      IF @V_IMPOSTO_IDTRATIMPO IN ('D','R') BEGIN
                         SET @V_VRIMPITM =@V_VRIMPITM - @V_IMPOSTO_VRIMPCOLETA;
                      END;
                   END;
                END;
             END;
             FETCH NEXT FROM C_IMPOSTO INTO @V_IMPOSTO_CDIMPOSTO,
                                            @V_IMPOSTO_IDTRATIMPO,
                                            @V_IMPOSTO_IDIMPORECUP,
                                            @V_IMPOSTO_CDCCTBLATIV,
                                            @V_IMPOSTO_CDCCTBLPASS,
                                            @V_IMPOSTO_VRADIMPCOL,
                                            @V_IMPOSTO_VRALIQIMPCOL,
                                            @V_IMPOSTO_VRIMPCOLETA,
                                            @V_IMPOSTO_CDFISCITMCOL;
          END;
          CLOSE C_IMPOSTO;
          DEALLOCATE C_IMPOSTO;
          SELECT
             @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
          FROM
             PRODUTO
          WHERE
             CDPRODUTO = @V_CDPRODUTO;
       END;

       UPDATE CUSTOSUP
       SET VRCUSPRLISTP = @V_MINPRECO,
           CDFILCOTACOR = @P_CDFILIAL,
           NRCOLEPRACOR = @V_NRCOLEPRECO,
           CDREGIAOACOR = @V_CDREGIAO,
           CDFORNECACOR = @V_CDFORNECED,
           CDPRODACOR   = @V_CDPRODUTO,
           CDMARCAACOR  = @V_CDMARCA,
           DTFINCUSTSUP = @ACORDO_DTFINVALITM,

           VRCUSPRLISM1 = (SELECT ISNULL(MAX(MOVINDICE.VRINDICE), 0) * ISNULL(@V_MINPRECO, 0)
                           FROM PARAMGERAL, MOVINDICE
                           WHERE @V_IDUTCUSTOM1 = 'S'  AND
                                 PARAMGERAL.CDINDICEMF = MOVINDICE.CDINDICE AND
                                 MOVINDICE.DTREFIND    = @ACORDO_DTINIVALITM AND
                                 MOVINDICE.CDTPINDICE  = '1' AND
                                 MOVINDICE.IDPERVALOR  = '2'),

           VRCUSPRLISM2 = (SELECT ISNULL(MAX(MOVINDICE.VRINDICE), 0) * ISNULL(@V_MINPRECO, 0)
                           FROM PARAMGERAL, MOVINDICE
                           WHERE @V_IDUTCUSTOM1 = 'S' AND
                                 PARAMGERAL.CDINDICEMF2 = MOVINDICE.CDINDICE AND
                                 MOVINDICE.DTREFIND     = @ACORDO_DTINIVALITM AND
                                 MOVINDICE.CDTPINDICE   = '1' AND
                                 MOVINDICE.IDPERVALOR   = '2'),

           VRCUSLIPRIMP = CASE @V_IDUTCUSLIPR
                         WHEN 'N' THEN NULL
                                  ELSE
                                  CASE @V_NRCOLEPRECO
                                  WHEN NULL THEN NULL
                                            ELSE
                                           ((@V_MINPRECO*@V_VRFATOCONV) + @V_VRIMPITM)/@V_VRFATOCONV
                                  END
                         END
       WHERE
         CDFILIAL   = @ACORDO_CDFILIAL   AND
         CDPRODUTO  = @ACORDO_CDPRODESTO AND
         DTCUSTOSUP = @ACORDO_DTINIVALITM;
    END;

    /* Update e Insert a seguir:
       Faz o tratamento dos custos de lista de preos que esto dentro
       do perodo de validade da cotao de acordo: esses custos so
       excludos pois o custo vlido deve ser o custo do acordo.
    */

    UPDATE CUSTOSUP
    SET
      VRHISPRLISTP = NULL,
      VRCUSPRLISTP = NULL,
      VRCUSPRLISM1 = NULL,
      VRCUSPRLISM2 = NULL,
      CDFILCOTLIST = NULL,
      NRCOLEPRLIST = NULL,
      CDMARCALIST  = NULL,
      CDREGIAOLIST = NULL,
      CDFORNECLIST = NULL,
      CDPRODLIST   = NULL
    WHERE
      CDFILIAL   = @ACORDO_CDFILIAL     AND
      CDPRODUTO  = @ACORDO_CDPRODESTO   AND
      DTCUSTOSUP > @ACORDO_DTINIVALITM  AND
      DTCUSTOSUP < @ACORDO_DTFINVALITM AND
      VRPRCUSTOAF IS NOT NULL;

    DELETE CUSTOSUP
    WHERE
      CDFILIAL   = @ACORDO_CDFILIAL    AND
      CDPRODUTO  = @ACORDO_CDPRODESTO  AND
      DTCUSTOSUP > @ACORDO_DTINIVALITM AND
      DTCUSTOSUP < @ACORDO_DTFINVALITM AND
      VRPRCUSTOAF IS NULL;
    FETCH NEXT FROM C_ACORDO INTO @ACORDO_CDFILIAL, @ACORDO_DTINIVALITM, @ACORDO_DTFINVALITM, @ACORDO_CDPRODESTO, @ACORDO_MINPRECO;
  END;
  CLOSE C_ACORDO;
  DEALLOCATE C_ACORDO;

  DELETE CUSTOSUP_TMP
  WHERE
    CDFILCENTCMP = @P_CDFILIAL AND
    NRSEQPRELIST = @P_NRSEQPRELIST;

  DELETE CUSTOSUPACOR_TMP
  WHERE
    CDFILCENTCMP = @P_CDFILIAL AND
    NRSEQPRELIST = @P_NRSEQPRELIST;

  IF (@P_CDFILCUSTO IS NULL) AND (@P_CDLISTAREG IS NULL) BEGIN
     DELETE FROM TEMPULTATULISPR
     WHERE CDCENULTLISPREC = @P_CDFILIAL
       AND NRSEQULTLISPREC = @P_NRSEQPRELIST;
  END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LISPREC_UPPRODFILI','6.19.XXX';
GO

